import React from 'react'
import { action as MetaAction, AppLoader } from 'edf-meta-engine'
import config from './config'
import { FormDecorator, LoadingMask, Icon } from 'edf-component'
import { fetch } from 'edf-utils'


class action {
	constructor(option) {
		this.metaAction = option.metaAction
		this.config = config.current
		this.webapi = this.config.webapi
		this.beforeLoad = option.voucherAction.excelbeforeUpload
		this.docBeforeLoad = option.voucherAction.docBeforeUpload
		this.download = option.voucherAction.download
	}

	onInit = ({ component, injections }) => {
		this.component = component
		this.injections = injections

		if (this.component.props.setOkListener) {
			this.component.props.setOkListener(this.onOk)
		}

		injections.reduce('init')
		this.load()
	}

	load = async () => {
		let res = await this.webapi.tranreport.queryEnclosure({id: this.component.props.id})
		this.injections.reduce('load', res)
	}

	//初始化附注内容
	getUploadContent = (type) => {
		let _this = this
		let content = [{
			name: 'title',
			component: '::p',
			className: 'title',
		}, {
			name: 'word',
			component: '::p',
			className: 'word',
		}, {
			name: 'upload',
			component: 'Upload',
			beforeUpload: type=='left' ? this.beforeLoad : this.docBeforeLoad,
			children: [{
			   name: 'openingBankItem',
			   className:'ant-btn-primary',
			   component: '::p',
			   children: '上传文件'//'{{data.file ? "重选文件" :"选择文件"}}'
			}],
			onChange: function(info){_this.uploadChange(info, type)},
			showUploadList: false,
			action: '/v1/edf/file/upload',
			headers: this.getAccessToken(),
			accept: type=='left' ? '.xls, .xlsx' : '.doc, .docx'
	   }, {
			name: 'uploadList',
			component: '::p',
			className: 'upload-list',
			children: this.getUploadList(type)

	   }]

		if(type == 'left'){
			content[0].children = '原财务报表(条件报送)'
			content[1].children = '注：格式只支持xls、xlsx，单个文件不超过10M。'
		}else{
			content[0].children = '附注资料(条件报送)'
			content[1].children = '注：格式只支持doc、docx，单个文件不超过10M。'
		}

		return content
	}

	//展示附件列表
	getUploadList = (type) => {
		let list = this.metaAction.gf('data.reportEnclosures').toJS(), uploadArr = []
		if(type!='left') list = this.metaAction.gf('data.nodeEnclosures').toJS()
		list.map((item)=>{
			uploadArr.push({
				name: 'uploadItem',
				component: '::p',
				children: [{
					name: 'filename',
					component: '::span',
					children: item.file.originalName
				}, this.getRightBtn(item, type)]
			})
		})
		return uploadArr
	}

	//展示右侧按钮
	getRightBtn = (item, type) => {
		let _this = this
		return <span className='right-btn'>
			<Icon type="xiazai"
				fontFamily='edficon' 
				title="下载"
				style={{color: '#87CF68'}}
				onClick={ function(){ _this.checkUploadFile(item, type)} } />
			<Icon type="shanchu"
				fontFamily='edficon'
				title="删除"
				onClick={ function(){ _this.deleteUploadFile(item, type)} } />
		</span>
	}

	//下载文件
	checkUploadFile = (option, type) => {
		this.download(option.file)
	}

	//删除上传文件
	deleteUploadFile = (option, type) => {
		this.injections.reduce('deleteUploadFile', option, type)
	}

	//上传文件
	uploadChange = (info, type) => {
		let tip = '仅支持上传Excel格式的文件'
		if(type != 'left') tip = '仅支持上传Doc格式的文件'
		if (!info.file.status) {
            this.metaAction.toast('error', tip)
            return
        }
		this.metaAction.sf('data.loading', true)
		if (info.file.status === 'done') {
			this.metaAction.sf('data.loading', false)
			if (info.file.response.error && info.file.response.error.message) {
				this.metaAction.toast('error', info.file.response.error.message)
			} else if (info.file.response.result && info.file.response.value) {
				this.injections.reduce('upload', info.file.response.value, type)
			}
		} else if (info.file.status === 'error') {
			this.metaAction.sf('data.loading', false)
			this.metaAction.toast('error', '上传失败')
		}
	}

	//获取token
	getAccessToken = () => {
		let token = fetch.getAccessToken()
		return { token }
	}

	onOk = async () => {
		return await this.save()
	}

	save = async () => {
		let option = {}, 
			reportEnclosures = this.metaAction.gf('data.reportEnclosures').toJS(), reportEnclosuresArr=[],
			nodeEnclosures = this.metaAction.gf('data.nodeEnclosures').toJS(), nodeEnclosuresArr=[],
			isOk = this.metaAction.gf('data.isOk')
		
		if (!isOk) {
			this.metaAction.sf('data.isOk', true)
			return false
		}

		option.id = this.component.props.id
		reportEnclosures.map((item)=> {
			reportEnclosuresArr.push({
				fileId: item.fileId,
				file: item.file
			})
		})
		option.reportEnclosures = reportEnclosuresArr
		nodeEnclosures.map((item)=> {
			nodeEnclosuresArr.push({
				fileId: item.fileId,
				file: item.file
			})
		})
		option.nodeEnclosures = nodeEnclosuresArr

		this.metaAction.sf('data.isOk', false)
		this.metaAction.sf('data.loading', true)
		option.isReturnValue = true			
		let response = await this.webapi.tranreport.uploadEnclosure(option)
		this.metaAction.sf('data.loading', false)

		if (response && response.result == false) {
			this.metaAction.toast('error', response.error.message)
			return false
		} else {
			this.metaAction.toast('success', `上传附件成功!`)
			this.metaAction.sf('data.isOk', true)
			return response
		}
	}
}

export default function creator(option) {
	const metaAction = new MetaAction(option),
		voucherAction = FormDecorator.actionCreator({ ...option, metaAction }),
		o = new action({ ...option, metaAction, voucherAction }),
		ret = { ...metaAction, ...voucherAction, ...o }
	metaAction.config({ metaHandlers: ret })

	return ret
}