import React from 'react'
import { action as MetaAction } from 'edf-meta-engine'
import { fromJS } from 'immutable'
import config from './config'
import { fetch, environment } from 'edf-utils'
import { Select, Button, Icon, FormDecorator, LoadingMask, Input } from 'edf-component'
import { Checkbox, Popover } from 'antd'

const Option = Select.Option;
/**
 * 
 * @param {text、select组件} param0 
 */
const ExtendEditableSelectCell = ({ dataSource, onChange, value, columnName, saveNewVlue, handleFoucus, disabled, tableType, getPopupContainer, divId }) => {
    let browserType = environment.getBrowserVersion()
    let productChildren = []
    //新增按钮 实现新增档案
    if (dataSource) {
        dataSource.forEach((item, index) => {
            if (typeof item == 'object') {
                productChildren.push(<Option title={item.title} value={item.fileNameInDto}>{item.title}</Option>)
            } else {
                productChildren.push(<Option title={item} value={item}>{item}</Option>)
            }
        })
    }
    return (
        browserType.ie ?
            <Select style={{ width: '100%', height: '26px'}}
                onChange={(value) => onChange(value)}
                value={(value && value.id) ? value.id : value}
                filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                getPopupContainer={() => getPopupContainer()}
                disabled={disabled}
                dropdownClassName='projectadjust-select-dropDown'
                onFocus={handleFoucus}>
                {productChildren}
            </Select>
            :
            <div style={{ width: '100%', height: '26px'}} id={divId} >
                <Select style={{ width: '100%' }}
                    value={(value && value.id) ? value.id : value}
                    onChange={(value) => onChange(value)}
                    filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
                    getPopupContainer={() => getPopupContainer()}
                    disabled={disabled}
                    //showSearch
                    onFocus={handleFoucus}>
                    {productChildren}
                </Select>
            </div>
    )
}
/**
 * 
 * @param {Checkbox 组件} param0 
 */
const EditableCheckbox = ({ checked, onChange }) => {
    return (
        <div style={{ 'display': 'block', 'width': '100%', 'textAlign': 'center' }}>
            <Checkbox
                onChange={onChange}
                checked={!!checked}
            ></Checkbox>
        </div>
    )
}

class action {
    constructor(option) {
        this.metaAction = option.metaAction
        this.voucherAction = option.voucherAction
        this.config = config.current
        this.webapi = this.config.webapi
    }
    onInit = ({ component, injections }) => {
        this.component = component
        this.injections = injections
        if (this.component.props.setOkListener) {
            this.component.props.setOkListener(this.onOk)
        }
        injections.reduce('init', {
            status: this.component.props.status,
            initData: this.component.props.initData
        })
        this.load()
    }

    load = async () => {
        let reportCode = this.metaAction.gf('data.form.activeTab') ? this.metaAction.gf('data.form.activeTab') : 'balancesheet'
        document.querySelector('.tranreport-projectadjust-modal .ant-modal-footer .ant-btn-primary').disabled = true
        this.queryTableList(reportCode)
    }
    rptTypeChange = (path, value) => {
        this.metaAction.sf(path, value)
    }
    queryTableList = async (reportCode) => {
        let res,
            paramsData = this.metaAction.gf('data.params'),
            params = paramsData ? {
                ...paramsData.toJS()
            } : {}
        res = await this.webapi.projectadjust.init(params)
        if (this.metaAction.gf('data.other.fromPromp')) {
            let firstReportCode
            if (res) {
                res.map(item => {
                    if (!firstReportCode && ((item.headList && item.headList.length) || (item.detailList && item.detailList.length))) {
                        firstReportCode = item.reportCode.toLocaleLowerCase()
                    }
                })
            }
            reportCode = firstReportCode ? firstReportCode : 'balancesheet'
        }
        this.injections.reduce('load', res, reportCode)
    }
    //获取需要编辑的列表
    getCurList = (tableType) => {
        let curTab = this.metaAction.gf('data.form.activeTab'),
            activeTab = curTab ? curTab : 'balancesheet',
            list,
            path,
            dataSourcePath,
            params = {}
        path = `data.${activeTab}${tableType}List`
        dataSourcePath = `data.${activeTab}${tableType}DataSource`
        params.path = path
        params.list = this.metaAction.gf(path) ? this.metaAction.gf(path).toJS() : []
        params.dataSource = this.metaAction.gf(dataSourcePath) ? this.metaAction.gf(dataSourcePath).toJS() : []
        return params
    }
    getPopupContainer = (index, tableType) => {
        let transformcss = tableType == 'Col' ? '.app-transform-left-rpttb' : '.app-transform-right-rpttb'
        return document.querySelector(transformcss)
    }
    /**
     * /渲染表格文本选择控件
     */
    renderCell = (columnName, index, value, tableType) => {
        let editable = true,
            curTab = this.metaAction.gf('data.form.activeTab')
        let activeTab = curTab ? curTab : 'balancesheet',
            params = this.getCurList(tableType),
            list = params.list
        if (list[index]['empty']) {
            return this.renderTextCell(columnName, index, value)
        } else {
            if (columnName == 'targetame') {
                if (!!list[index]['ignore']) {
                    return (<Input style={{ width: '100%', height: '26px' }} disabled={true} />)
                } else {
                    return (<ExtendEditableSelectCell
                        editable={editable}
                        onChange={(value) => this.handleSelectChange(value, columnName, index, params.dataSource, tableType)}
                        dataSource={params.dataSource}
                        value={value}
                        divId={`${tableType}${index}`}
                        columnName={columnName}
                        disabled={!!list[index]['ignore']}
                        tableType={tableType}
                        getPopupContainer={() => this.getPopupContainer(index, tableType)}
                    />)
                }
            } else if (columnName == 'ignore') {
                return (<EditableCheckbox
                    checked={!!value}
                    onChange={(value) => this.handleChange(value, columnName, index, tableType)}
                />)
            } else if (columnName == 'sourceName') {
                return this.renderTextCell(columnName, index, value, list[index]['ignore'], list[index]['new'], tableType)
            }
        }
    }
    /**
     * 渲染表格原表所在列图标
     */
    renderTextCell = (columnName, index, value, ignore, newStr, tableType) => {
        const styleName = tableType == 'Col' ? 'tranreport-projectadjust-colcell' : 'tranreport-projectadjust-rowcell'
        return (<div className={ignore ? `${styleName} cell-color-grid` : `${styleName}`}>
            {(value && value != undefined) ? <span title={value}>{value}</span> : ''}
            {(columnName == 'sourceName' && newStr == true) ? <img src={require('./img/new.gif')} style={{
                width: '29px', height: '11px', 'minWidth': '29px', position: 'relative'}} /> : ''}
        </div>)
    }
    /**
     * 忽略单击触发得事件
     */
    handleChange = (value, columnName, index, tableType) => {
        let params = this.getCurList(tableType),
            list = fromJS(params.list),
            oldList = params.list,
            path = params.path
        let values
        list = list.update(index, item => item.set('ignore', value.target.checked))
        list = list.update(index, item => item.set('targetame', undefined))
        list = list.update(index, item => item.set('targeFieldName', undefined))
        this.updateTableItem(tableType, list.toJS(), oldList, path, index)
    }
    getLastTitle = (text, eleClassName) => {
        return <div>
            <span>忽略</span><Popover content={text} placement="left" getTooltipContainer={() => document.querySelector(eleClassName)}>
                <Icon className='ignore-title-icon' />
            </Popover>
        </div>
    }
    /**
     * 选择框值改变触发
     */
    handleSelectChange = async (value, columnName, index, dataSource, tableType) => {
        let params = this.getCurList(tableType),
            list = fromJS(params.list),
            oldList = params.list,
            path = params.path
        if (tableType == 'Col') {
            const selectObj = dataSource.filter((item) => item.fileNameInDto == value)
            list = list.update(index, item => item.set('targetame', selectObj[0].title))
            list = list.update(index, item => item.set('targeFieldName', selectObj[0].fileNameInDto))
        } else if (tableType == 'Row') {
            list = list.update(index, item => item.set(columnName, value))

        }
        //改值实时调接口
        this.updateTableItem(tableType, list.toJS(), oldList, path, index)
    }
    /**
     * 实时调用接口保存下拉控件修改得值
     */
    updateTableItem = async (tableType, list, oldList, path, index) => {

        let curTab = this.metaAction.gf('data.form.activeTab'),
            activeTab = curTab ? curTab : 'balancesheet'

        document.querySelector('.tranreport-projectadjust-modal .ant-modal-footer .ant-btn-primary').disabled = false

        let data = {}
        data[`data.other.${activeTab}EditStatus`] = true
        data[path] = fromJS(list)
        this.metaAction.sfs(data)
    }
    /**
     * 重新转换
     */
    onOk = async () => {
        return await this.save()
    }
    save = async () => {
        let checkResponse = this.checkLists(),
            res2
        if (!checkResponse.responseStatus) {
            let res1 = await this.metaAction.modal('confirm', {
                content: checkResponse.messageInfo1.join('、') + '存在无对应关系的原表' + checkResponse.messageInfo2.join('、') + '，将被忽略掉。您可以取消忽略并为其选择匹配的局端' + checkResponse.messageInfo2.join('、') + '。',
                className: 'tranreport-prompt-modal',
                okText: '重新转换',
                cancelText: '修改匹配项'
            })
            if (!res1) {
                return false
            }
        }
        let otherData = this.metaAction.gf('data.other')
        let paramsData = this.metaAction.gf('data.params'),
            params = paramsData ? {
                calcBalanceSheet: !!otherData.get('balancesheetEditStatus'),
                calcProfitStatement: !!otherData.get('profitstatementEditStatus'),
                calcCashFlowStatement: !!otherData.get('cashflowstatementEditStatus'),
                calcSumRow: true,
                headList: checkResponse.colSuccessList,
                detailList: checkResponse.rowSuccessList,
                ...paramsData.toJS()
            } : {}

        let res = await this.webapi.projectadjust.matchingBatch(params)
        if (res) {
            delete params.headList
            delete params.detailList
            res2 = await this.webapi.projectadjust.calcReport(params)
            if (res2) {
                return res2
            }
        }
    }
    checkLists = () => {
        let tableName = ['balancesheet', 'profitstatement', 'cashflowstatement'],
            checkResponse = true,
            data = this.metaAction.gf('data'),
            colIgnoreList = [], rowIgnoreList = [], messageInfo1 = [], messageInfo2 = [],
            colSuccessList = [],
            rowSuccessList = [],
            editItems = []

        tableName.map(item => {
            let rescol = {}, resrow = {}
            if (this.metaAction.gf(`data.${item}ColState`)) {
                let colList = this.metaAction.gf(`data.${item}ColList`).toJS()
                colList = colList.slice(0, this.metaAction.gf(`data.${item}ColListLength`))
                rescol = this.checkListData(colList, 'col')
                if (!rescol.state) {
                    colIgnoreList = colIgnoreList.concat(rescol.data)
                }
                colSuccessList = colSuccessList.concat(rescol.successData)
                if (checkResponse) {
                    checkResponse = rescol.state
                }
            }
            if (this.metaAction.gf(`data.${item}RowState`)) {
                let rowList = this.metaAction.gf(`data.${item}RowList`).toJS()
                rowList = rowList.slice(0, this.metaAction.gf(`data.${item}RowListLength`))
                resrow = this.checkListData(rowList, 'row')
                if (!resrow.state) {
                    rowIgnoreList = rowIgnoreList.concat(resrow.data)
                }
                rowSuccessList = rowSuccessList.concat(resrow.successData)
                if (checkResponse) {
                    checkResponse = resrow.state
                }
            }
            if (resrow.state === false || rescol.state === false) {
                editItems.push(item)
                messageInfo1.push(
                    (item == 'balancesheet' ? '资产负债表' : (item == 'profitstatement' ? '利润表' : '现金流量表'))
                )
            }
        })
        if (rowIgnoreList.length) {//有没忽略的行数据 提示信息中加 行
            messageInfo2.push('行')
        }
        if (colIgnoreList.length) {//有没忽略的列数据 提示信息中加 列
            messageInfo2.push('列')
        }
        return {
            responseStatus: checkResponse,
            colIgnoreList,
            rowIgnoreList,
            colSuccessList: colSuccessList.concat(colIgnoreList),
            rowSuccessList: rowSuccessList.concat(rowIgnoreList),
            editItems,
            messageInfo1,//提示信息包含的报表名
            messageInfo2//提示信息包含的行或列
        }
    }
    checkListData = (list, type) => {
        let checkResponse = true,
            arr = [],
            successArr = []
        list.map(item => {
            if (!item.targetame && !item.ignore) {
                checkResponse = false
                delete item.new
                delete item.ts
                delete item.orgId
                item.ignore = 1
                if (type == 'col') {
                    item.targeFieldName = null
                    item.targetame = null
                } else if (type == 'row') {
                    item.targetame = null
                }
                arr.push(item)
            } else {
                item.ignore = item.ignore ? 1 : 0
                delete item.ts
                delete item.orgId
                delete item.new
                if (type == 'col') {
                    if (!item.targeFieldName) {
                        item.targeFieldName = null
                    }
                    if (!item.targetame) {
                        item.targetame = null
                    }
                } else if (type == 'row') {
                    if (!item.targetame) {
                        item.targetame = null
                    }
                }
                successArr.push(item)
            }
        })
        return {
            state: checkResponse,
            data: arr,
            successData: successArr
        }
    }
    getTableTitle = () => {
        return <p style={{ 'marginBottom': '0px' }} style={{ width: '100%', 'marginBottom': 0 }}>
            行：<span className="tranreport-projectadjust-table-right-title" style={{ 'color': '#FF7E3B', 'float': 'right', 'fontSize': '12px' }}>注：下方表格只显示不一致的行。</span>
        </p>
    }
}

export default function creator(option) {
    const metaAction = new MetaAction(option),
        o = new action({ ...option, metaAction }),
        ret = { ...metaAction, ...o }
    metaAction.config({ metaHandlers: ret })
    return ret
}
