import React from 'react'
import { Map, fromJS } from 'immutable'
import { action as MetaAction, AppLoader } from 'edf-meta-engine'
import { TableOperate, Select, LoadingMask, Button, Tooltip, Modal, Input, Checkbox, Icon, Upload, Radio } from 'edf-component'
import extend from './extend'
import config from './config'
import moment from 'moment'
import * as data from './data'
import { fetch, number, history } from 'edf-utils'
import { debug } from 'util';

class action {
    constructor(option) {
        this.metaAction = option.metaAction
        this.extendAction = option.extendAction
        this.config = config.current
        this.webapi = this.config.webapi
    }
    //初始化
    onInit = async ({ component, injections }) => {
        this.extendAction.gridAction.onInit({ component, injections })
        this.component = component
        this.injections = injections
        if (this.component.props.setOkListener) {
            this.component.props.setOkListener(this.onOk)
        }
        injections.reduce('init')
        this.load(this.component.props.initData)
    }
    load = async (option) => {
        this.injections.reduce('load', option)
    }
    /**
     * 渲染复选框
     */
    renderCell = (text, record, index) => {
        const selected = record && record.selected ? true : false
        return (<EditableCheckbox
            checked={selected}
            disabled={false}
            onChange={(value) => this.handleChange(value, index)}
        />)
    }
    //checkbox 选择事件
    handleChange = (value, index) => {
        let list = this.metaAction.gf('data.list')
        list = list.update(index, item => item.set('selected', value.target.checked))
        this.metaAction.sf('data.list', list)
    }

    //校验单数据
    checkSaveData = () => {
        const list = this.metaAction.gf('data.list').toJS()
        let arr = []
        list.map(item => {
            if (item.selected) {
                arr.push(item)
            }
        })
        return arr
    }
    //保存
    onOk = async () => {
        return await this.save()
    }
    //选单保存
    save = async () => {
        const checkVaild = this.checkSaveData()
        if (checkVaild.length < 1) {
            this.metaAction.toast('success', `请至少选择一张报表!`)
            return false
        }
        const list = this.metaAction.gf('data.list').toJS(),
            appraisalId = this.metaAction.gf('data.appraisalId')
        this.metaAction.sf('data.other.isOk', false)
        let response = await this.webapi.rptselect.saveSelectReport({ list, appraisalId })        
        if (response) {
            this.metaAction.toast('success', `保存成功!`)
            this.metaAction.sf('data.other.isOk', true)
            return response
        } else {            
            this.metaAction.sf('data.other.isOk', true)
            return false
        }
    }
}

export default function creator(option) {
    const metaAction = new MetaAction(option),
        extendAction = extend.actionCreator({ ...option, metaAction }),
        o = new action({ ...option, metaAction, extendAction }),
        ret = { ...metaAction, ...extendAction.gridAction, ...o }
    metaAction.config({ metaHandlers: ret })
    return ret
}
/**
 * 
 * @param {Checkbox 组件} param0 
 */
const EditableCheckbox = ({ checked, onChange }) => {
    return (
        <div style={{ 'display': 'block', 'width': '100%', 'textAlign': 'center' }}>
            <Checkbox
                onChange={onChange}
                checked={!!checked}
            ></Checkbox>
        </div>
    )
}