/**
 * mock.js 提供应用截获ajax请求，为脱离后台测试使用
 * 模拟查询更改内存中mockData,并返回数据
 */

import { fetch } from 'edf-utils'

const mockData = fetch.mockData

function initUnit() {
    if(!mockData.unit){
        mockData.unit = [{
			"id": 1,
			"code": "001",
			"name": "个",
			"helpCode": "G"
		}, {
			"id": 2,
			"code": "002",
			"name": "台",
			"helpCode": "T"
		}, {
			"id": 3,
			"code": "003",
			"name": "件",
			"helpCode": "J"
		}, {
			"id": 4,
			"code": "004",
			"name": "只",
			"helpCode": "Z"
		}, {
			"id": 5,
			"code": "005",
			"name": "根",
			"helpCode": "G"
		}, {
			"id": 7,
			"code": "G",
			"name": "克",
			"helpCode": "K"
		}, {
			"id": 8,
			"code": "Kg",
			"name": "千克",
			"helpCode": "QK"
		}, {
			"id": 6,
			"code": "M",
			"name": "米",
			"helpCode": "M"
		}, {
			"id": 9,
			"code": "T",
			"name": "吨",
			"helpCode": "D"
		}]
    }
}

function initCurrency() {
    if(!mockData.currency){
        mockData.currency = [{
			"exchangeRate": 1,
			"isBaseCurrency": true,
			"id": 1,
			"code": "RMB",
			"name": "人民币",
			"helpCode": "RMB",
			"isEnable": true
		}, {
			"exchangeRate": 6.7705,
			"isBaseCurrency": false,
			"id": 2,
			"code": "USD",
			"name": "美元",
			"helpCode": "MY",
			"isEnable": true
		}, {
			"exchangeRate": 7.3707,
			"isBaseCurrency": false,
			"id": 3,
			"code": "EUR",
			"name": "欧元",
			"helpCode": "OY",
			"isEnable": true
		}, {
			"exchangeRate": 0.065026,
			"isBaseCurrency": false,
			"id": 4,
			"code": "JPY",
			"name": "日元",
			"helpCode": "RY",
			"isEnable": true
		}, {
			"exchangeRate": 8.2472,
			"isBaseCurrency": false,
			"id": 5,
			"code": "GBP",
			"name": "英镑",
			"helpCode": "YB",
			"isEnable": true
		}, {
			"exchangeRate": 3.8089,
			"isBaseCurrency": false,
			"id": 6,
			"code": "DEM",
			"name": "德国马克",
			"helpCode": "DGMK",
			"isEnable": true
		}, {
			"exchangeRate": 5.1803,
			"isBaseCurrency": false,
			"id": 7,
			"code": "AUD",
			"name": "澳大利亚元",
			"helpCode": "ADLYY",
			"isEnable": true
		}, {
			"exchangeRate": 0.8729,
			"isBaseCurrency": false,
			"id": 8,
			"code": "HKD",
			"name": "港币",
			"helpCode": "GY",
			"isEnable": true
		}]
    }
}

fetch.mock('v1/ba/unit/queryList', (option) => {
    initUnit()
    return {result:true, value: {list: mockData.unit}}
})

fetch.mock('/v1/ba/currency/queryList', (option) => {
    initCurrency()
    return {result:true, value: {list: mockData.currency}}
})

fetch.mock('/v1/gl/account/update', (option) => {
    update(option.glAccountDto, 'update')
    return {result:true, value:{}}
})

fetch.mock('/v1/gl/account/getById', (option) => {
    let value = {
        calcDict: mockData.calcDict, 
        glAccount: query(option)
    }
    return {result:true, value: value}
})

fetch.mock('/v1/gl/account/create', (option) => {
    update(option, 'add')
    
    return {result:true, value:{}}
})

fetch.mock('/v1/gl/account/isUsedInCertificate', (option) => {
    return {result:true, value:false}
})

function query(option){
    let glAccounts =  mockData.glAccounts,
        subject = glAccounts.filter(item => item.id == option.id)
    return subject[0]
}

function update(option, state){
	function build(list, option, state){  
		let{ accountTypeId, parentId, id, code, name, grade, 
			isCalcQuantity, isCalcMulti, balanceDirection, balanceDirectionName,
			isCalc, isCalcDepartment, isCalcPerson, isCalcCustomer, isCalcSupplier,
			isCalcInventory, isCalcProject, isCalcBankAccount} = option
		option.children = []
		list.map((item) => {
			if(!item.grade || item.grade < grade){
				if(item.id == parentId){
					if(state == 'add'){
						option.codeAndName = item.codeAndName + '-' + name
						option.id = item.id + option.code.substring(option.code.length - 2)
						item.isEndNode = false
						item.children.push(option)
					}else if(state == 'update'){
						item.children.map((o) => {
							if(o.id == id){
								o.code = code
								o.name = name
								o.isCalcMulti = isCalcMulti
								o.isCalcQuantity = isCalcQuantity
								o.balanceDirection = balanceDirection
								o.balanceDirectionName = balanceDirectionName
								o.isCalc = isCalc
								o.isCalcDepartment = isCalcDepartment
								o.isCalcPerson = isCalcPerson
								o.isCalcCustomer = isCalcCustomer
								o.isCalcSupplier = isCalcSupplier
								o.isCalcInventory = isCalcInventory
								o.isCalcProject = isCalcProject
								o.isCalcBankAccount = isCalcBankAccount
								if(isCalcMulti){
									o.currencyId = option.currencyId
									o.currencyName = option.currencyName
								}else{
									delete o.currencyId
									delete o.currencyName
								}
								if(isCalcQuantity){
									o.unitName = option.unitName
									o.unitId = option.unitId
								}else{
									delete o.unitName
									delete o.unitId
								}
							}
						})
					}
				}else if(item.children.length != 0){
					build(item.children, option, state)
				}
			}
		})
	}

	build(mockData.stockTypes, option, state)
}

  