import React from 'react'
import ReactDOM from 'react-dom'
import config from './config'
import moment from 'moment'
import extend from './extend'
import * as data from './data'
import utils, { fetch } from 'edf-utils'
import { action as MetaAction, AppLoader, } from 'edf-meta-engine'
import { Popover, Button, Link } from 'antd'
import { PrintOption, Modal } from 'edf-component'
import { Map, fromJS, List, is } from 'immutable'
import { consts } from 'edf-consts'

class action {
    constructor(option) {
        this.metaAction = option.metaAction
        this.extendAction = option.extendAction
        this.config = config.current
        this.webapi = this.config.webapi
    }

    onInit = ({ component, injections }) => {
        this.extendAction.gridAction.onInit({ component, injections })
        this.component = component
        this.injections = injections
        injections.reduce('init')

        let initData = this.component.props.initData
        let addEventListener = this.component.props.addEventListener

        if (addEventListener) {
            addEventListener('onTabFocus', :: this.onTabFocus)
        }

        this.setrowsCount()
        this.load(initData)
    }

    load = async (initData) => {
        const subjectList = await this.getSubjectList()
        const currencyList = await this.webapi.currency.queryList({entity: {isEnable:true}})
        const enabledYearMonth = this.getEnabledYearMonth()

        if (!initData) {
            let initCertificateData = await this.webapi.certificate.init({ isReturnValue: true })

            if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
                this.metaAction.toast('error', initCertificateData.error.message)
                this.injections.reduce('initNewCertificate', undefined, subjectList, enabledYearMonth, currencyList.list)
            } else {
                this.injections.reduce('initNewCertificate', initCertificateData, subjectList, enabledYearMonth, currencyList.list)
                this.extendAction.gridAction.cellAutoFocus()
            }
        } else if (initData.id) {
            const certificateData = await this.webapi.certificate.findById({ docId: initData.id })

            this.injections.reduce('initLoadCertificate', fromJS(certificateData), enabledYearMonth, subjectList, currencyList.list)
        //凭证管理中插入凭证时
        } else if (initData && initData.code) {
            if (initData.code == '9999') {
                this.metaAction.toast('error', '凭证号达到9999，不能再增加凭证！')
                return
            }
            this.checkEditStatusBeforeLoadNewVoucher(() => {
                initData.isInsert = true
                this.injections.reduce('clearAndNewCertificate', initData, subjectList, currencyList.list, enabledYearMonth)
            })
        //凭证管理中新增凭证时
        } else if (initData && initData.newCertificate) {
            this.checkEditStatusBeforeLoadNewVoucher(async () => {
                let initCertificateData = await this.webapi.certificate.init({ isReturnValue: true })

                if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
                    this.metaAction.toast('error', initCertificateData.error.message)
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList, currencyList.list, enabledYearMonth)
                } else {
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList, currencyList.list, enabledYearMonth)

                    this.extendAction.gridAction.cellAutoFocus()
                }
            })
        }

        //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
        //this.component.props.editing(this.component.props.appName, false);
        this.editCloseTips(false)
    }

    //当前app的 "tab被点击" (从其他app切换到当前app)
    onTabFocus = async (props) => {
        let initData = props ? props.initData : undefined
        const subjectList = await this.getSubjectList()
        const currencyList = await this.webapi.currency.queryList({entity: {isEnable:true}})

        //查看凭证
        if (initData && initData.id) {
            this.checkEditStatusBeforeLoadNewVoucher(async () => {
                let certificateData = await this.webapi.certificate.findById({ docId: initData.id })

                this.injections.reduce('initLoadCertificate', fromJS(certificateData), undefined, subjectList, currencyList.list)
            })
        }
        //凭证管理中插入凭证时
        else if (initData && initData.code) {
            if (initData.code == '9999') {
                this.metaAction.toast('error', '凭证号达到9999，不能再增加凭证！')
                return
            }
            this.checkEditStatusBeforeLoadNewVoucher(() => {
                initData.isInsert = true
                this.injections.reduce('clearAndNewCertificate', initData, subjectList, currencyList.list)
                this.extendAction.gridAction.cellAutoFocus()
            })
        //凭证管理中新增凭证时
        } else if (initData && initData.newCertificate) {
            this.checkEditStatusBeforeLoadNewVoucher(async () => {
                let initCertificateData = await this.webapi.certificate.init({ isReturnValue: true })

                if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
                    this.metaAction.toast('error', initCertificateData.error.message)
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList, currencyList.list)
                } else {
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList, currencyList.list)
                    this.extendAction.gridAction.cellAutoFocus()
                }
            })
        } else {
            let details = this.metaAction.gf('data.form.details'),
                editStatus = this.metaAction.gf('data.other.editStatus')

            this.upgradeDetails(details, subjectList, fromJS(currencyList.list))
            if (editStatus == data.VIEW_STATUS) {

                let id = this.metaAction.gf('data.form.id')

                const certificateData = await this.webapi.certificate.findById({ docId: id, isReturnValue: true })

                if (certificateData.result == false &&
                    certificateData.error.code == data.NOT_FOUND_CERTIFICATE) {

                    this.newCertificate(subjectList)
                } else {
                    const enabledYearMonth = this.getEnabledYearMonth()
                    this.injections.reduce('initLoadCertificate', fromJS(certificateData), enabledYearMonth, subjectList, currencyList.list)
                }
            }

        }
    }

    //关闭附件时机
    windowClick = (e) => {
        if (!document.getElementsByClassName('fixedDataTableLayout_rowsContainer')) return
        this.injections.reduce('attachmentVisible', false)
    }

    componentDidMount = () => {
        const win = window
        if (win.addEventListener) {
            document.getElementById("app").addEventListener('click', this.windowClick, false)
            document.body.addEventListener('keydown', this.bodyKeydownEvent, false)
        } else if (win.attachEvent) {
            document.getElementById("app").attachEvent('onclick', this.windowClick)
            document.body.attachEvent('onkeydown', this.bodyKeydownEvent)
        }
    }

    bodyKeydownEvent = (e) => {
        const dom = document.getElementById('ttk-voucher-app-document')
        const modalBody = document.getElementsByClassName('ant-modal-body')
        if (dom && modalBody && modalBody.length<1){
            this.keyDownCickEvent({event:e})
        }
    }

    onMouseDown = (e) => {
        const path = utils.path.findPathByEvent(e)
        if (path.indexOf('cell.cell.dropdownFooter.add') > -1) return true
        if (path.indexOf('.cell.cell,') != -1) {
            this.extendAction.gridAction.mousedown(e)
        }
    }

    getSubjectList = async () => {
        let subjectList = await this.webapi.accountingSubject.query({ isEndNode: true }),
            subjectListEnable = []

        subjectList.glAccounts.map(item => {
            if (!!item.isEnable) {
                subjectListEnable.push(item)
            }
        })

        subjectList.glAccountsAll = subjectList.glAccounts
        subjectList.glAccounts = subjectListEnable

        return subjectList
    }

    setrowsCount = () => {
        let charge = document.getElementsByClassName("edfx-app-portal-content-main"),
            data = this.metaAction.gf('data').toJS(),
            details = data.form.details,
            length = details.length,
            height, value = []

        if(charge.length != 0){
            height = charge[0].clientHeight - 311
            length = Math.ceil(height/60) > 5 ? Math.ceil(height/60) : 5
            const blankVoucherItem = {
                summary: '',
                accountingSubject: undefined,
                debitAmount: undefined,
                creditAmount: ''
            }
            value = Array(length - 5).fill(blankVoucherItem);
        }
        details = details.concat(value)
        this.injections.reduce('setrowsCount',details, details.length)
    }

    //监听键盘事件
    keyDownCickEvent = (keydown) => {
        if (keydown && keydown.event) {
            let e = keydown.event

            if (e.ctrlKey && e.altKey && (e.key == 'n' || e.keyCode == 78)) {
                if (this.isDisplayButton('add')) {
                    //新增
                    this.newCertificate()
                }
                if (e.preventDefault) {
                    e.preventDefault()
                }
                if (e.stopPropagation) {
                    e.stopPropagation()
                }
            }
            else if (e.ctrlKey && !e.altKey && (e.key == 's' || e.keyCode == 83)) {
                //保存 防止并发
                if (!this.metaAction.gf('data.other.btnStatus') && this.isDisplayButton('save')) {
                    this.backCaculateAmount()
                    this.save()
                }
                if (e.preventDefault) {
                    e.preventDefault()
                }
                if (e.stopPropagation) {
                    e.stopPropagation()
                }
            }
            else if (e.ctrlKey && !e.altKey && (e.key == '/' || e.keyCode == 191)) {
                //保存并新增 防止并发
                if (!this.metaAction.gf('data.other.btnStatus') && this.isDisplayButton('save')) {
                    this.backCaculateAmount()
                    this.saveAndNew()
                }
                if (e.preventDefault) {
                    e.preventDefault()
                }
                if (e.stopPropagation) {
                    e.stopPropagation()
                }
            }
            else if (e.ctrlKey && !e.altKey && (e.key == 'y' || e.keyCode == 89)) {
                //审核
                if (this.isDisplayButton('audit')) {
                    this.audit()
                }
                if (e.preventDefault) {
                    e.preventDefault()
                }
                if (e.stopPropagation) {
                    e.stopPropagation()
                }
            }
            else if (e.ctrlKey && !e.altKey && (e.key == ";" || e.keyCode == 186)) {
                //打印
                if (this.isDisplayButton('print')) {
                    this.print()
                }
            }
            else if ((e.key == "Enter" || e.keyCode == 13) &&
                this.metaAction.gf('data.other.focusFieldPath') == 'root.children.formHeader.children.date.children.date') {
                this.metaAction.sf('data.other.focusFieldPath', 'root.children.formHeader.children.attachment.children.attachment')
            }
            //判断设备是否为mac
            else if (navigator.userAgent.indexOf('Mac OS X') !== -1) {
                if (!e.ctrlKey && !e.altKey && (e.key == "" || e.keyCode == 8 || e.keyCode == 46)) {
                    //删除
                    if (!this.metaAction.gf('data.other.btnStatus') && !this.isDisplayButton('del')) {
                        this.del()
                    }
                }
                else if (e.ctrlKey && !e.altKey && (e.key == "[" || e.keyCode == 219)) {
                    //上一张
                    this.loadPrevCertificate()
                }

                else if (e.ctrlKey && !e.altKey && (e.key == "]" || e.keyCode == 221)) {
                    //下一张
                    this.loadNextCertificate()
                }
            } else {
                if (!e.ctrlKey && !e.altKey && (e.key == "Delete" || e.keyCode == 46)) {
                    //删除
                    if (!this.metaAction.gf('data.other.btnStatus') && !this.isDisplayButton('del')) {
                        this.del()
                    }
                }
                else if (e.ctrlKey && !e.altKey && (e.key == "[" || e.keyCode == 37 || e.keyCode == 219)) {
                    //219 win7 IE11下的keyCode
                    //上一张
                    this.loadPrevCertificate()
                }
                else if (e.ctrlKey && !e.altKey && (e.key == "]" || e.keyCode == 39 || e.keyCode == 221)) {
                    //221 win7 IE11下的keyCode
                    //下一张
                    this.loadNextCertificate()
                }
            }
        }
    }

    //光标闪烁时不起作用
    componentWillReceiveProps = ({ keydown }) => {
        // this.keyDownCickEvent(keydown)
    }

    selfGridKeydown = (e) => {
      if (document.getElementsByClassName('accountQuantityEdit').length > 0 &&
          document.activeElement.className.indexOf('ant-select-selection') > -1) {

         // 当有数量外币框时，并且焦点在币种框上时，将不执行grid的gridKeydown
         return
      }
      this.keyDownCickEvent({event: e})
      this.extendAction.gridAction.gridKeydown(e)
      if (e.keyCode == 40) { //40:向下键keycode
          this.extendAction.gridAction.cellAutoFocus()
      }
    }

    // 升级科目信息
    upgradeDetails = (details, subjectList, currencys) => {
        let curAccountingSubject, lastestSubject, resCurrency, quantityAndForeignCurrency,
            glAccounts = fromJS(subjectList.glAccounts),
            editStatus = this.metaAction.gf('data.other.editStatus')

        this.metaAction.sf('data.other.calcDict', Map(subjectList.calcDict))
        this.metaAction.sf('data.other.accountingSubjects', fromJS(subjectList.glAccounts))
        this.metaAction.sf('data.other.accountingSubjectsAll', fromJS(subjectList.glAccountsAll))
        this.metaAction.sf('data.other.currencyDS', currencys)

        for (var i = 0; i < details.size; i++) {
            curAccountingSubject = details.get(i).get('accountingSubject')
            /*场景描述：解决页签切换，停用币种的问题，先录入分录，然后在币种档案停用对应的币种，页签切换回来后更新停用的币种，把停用的币种清空
            */
            quantityAndForeignCurrency = details.get(i).get('quantityAndForeignCurrency') ? details.get(i).get('quantityAndForeignCurrency').toJS() : undefined
            if (currencys && curAccountingSubject) {
                if (quantityAndForeignCurrency && quantityAndForeignCurrency.currency){
                    if (quantityAndForeignCurrency.currency.id != curAccountingSubject.get('currencyId')) {
                        resCurrency = currencys.find(x => x.get('id') == quantityAndForeignCurrency.currency.id)
                    } else {
                        resCurrency = currencys.find(x => x.get('id') == curAccountingSubject.get('currencyId'))
                    }
                }
            }

            //页签切换,查找币种是否存在
            if (quantityAndForeignCurrency && !resCurrency) {
                quantityAndForeignCurrency.currency = null
                details = details.update(i, item => item.set('quantityAndForeignCurrency', fromJS(quantityAndForeignCurrency)))
            }

            if (curAccountingSubject != undefined) {
                lastestSubject = glAccounts.filter(item => {
                    return item.get('id') == curAccountingSubject.get('id')
                }).toArray()

                // 余额的保持
                if (editStatus == data.ADD_STATUS && curAccountingSubject.get('balance') != undefined) {
                    lastestSubject[0] = lastestSubject[0].set('balance', curAccountingSubject.get('balance')).set('initBalance', curAccountingSubject.get('balance'))
                }

                // 若最新科目去除所有辅助核算
                if (!!curAccountingSubject.get('isCalc') && lastestSubject[0] && !lastestSubject[0].get('isCalc') ||
                    !curAccountingSubject.get('isCalc') && lastestSubject[0] && !!lastestSubject[0].get('isCalc')) {

                    curAccountingSubject = lastestSubject[0]
                    details = details.update(i, item => item.set('accountingSubject', lastestSubject[0]))
                } else if (!!curAccountingSubject.get('isCalc') && lastestSubject[0] && !!lastestSubject[0].get('isCalc')) {
                    let auxAccountSubjects = curAccountingSubject.get('auxAccountSubjects'),
                        auxAccountSubjectsPreSelected = curAccountingSubject.get('auxAccountSubjectsPreSelected')

                    lastestSubject[0] = lastestSubject[0].set('auxAccountSubjects', auxAccountSubjects)
                    lastestSubject[0] = lastestSubject[0].set('auxAccountSubjectsPreSelected', auxAccountSubjectsPreSelected)
                    details = details.update(i, item => item.set('accountingSubject', lastestSubject[0]))
                } else if (!curAccountingSubject.get('isCalc') && lastestSubject[0] && !lastestSubject[0].get('isCalc')) {
                    details = details.update(i, item => item.set('accountingSubject', lastestSubject[0]))
                }
            }
        }
        this.metaAction.sf('data.form.details', details)
    }

    getEnabledYearMonth = () => {
        let currentOrg = this.metaAction.context.get("currentOrg")
        let enabledYearMonth = currentOrg.enabledYear + '-' + currentOrg.enabledMonth.toString().padStart(2, '0')

        return enabledYearMonth
    }

    getCurrentUser = () => {
        return this.metaAction.context.get("currentUser")
    }

    isDisplayButton = (btnName) => {
        let editStatus = this.metaAction.gf('data.other.editStatus'),
            certificateStatus = this.metaAction.gf('data.form.certificateStatus'),
            isDisplay = true

        if (btnName == 'add') {
            isDisplay = (editStatus == data.VIEW_STATUS ? true : false)
        } else if (btnName == 'del') {
            isDisplay = ((certificateStatus == data.STATUS_VOUCHER_AUDITED) ||
                (certificateStatus == data.STATUS_VOUCHER_NOT_AUDITED &&
                    editStatus != data.VIEW_STATUS))
        } else if (btnName == 'save') {
            isDisplay = (editStatus != data.VIEW_STATUS ? true : false)
        } else if (btnName == 'audit') {
            isDisplay = (editStatus == data.VIEW_STATUS ? true : false)
        } else if (btnName == 'print') {
            isDisplay = (editStatus == data.VIEW_STATUS ? true : false)
        } else if (btnName == 'more') {
            isDisplay = (editStatus != data.ADD_STATUS ? true : false)
        }

        return isDisplay
    }

    getDynamicComponent = (_ctrlPath) => {
        let _isReadOnly = this.metaAction.isReadOnly(_ctrlPath),
            _isAccountFocus = this.isFocusOnAccount(_ctrlPath),
            _focusFieldPath = this.metaAction.gf('data.other.focusFieldPath'),
            _cancelEditAuxAccount = this.metaAction.gf('data.other.cancelEditAuxAccount'),
            _rowIndex = 0
        if (_focusFieldPath) {
            if (_focusFieldPath.indexOf('.cell.cell') != -1) {
                _rowIndex = _focusFieldPath.split('.cell.cell,')[1]
            }
        } else {
            return 'SubjectDisplay'
        }

        if (_ctrlPath) {
            if (_ctrlPath.indexOf('.cell.cell,' + _rowIndex) > -1) {
                if (_isReadOnly) {
                    return 'SubjectDisplay'
                }
                else {
                    if (_isAccountFocus || _cancelEditAuxAccount) {
                        this.metaAction.sf('data.other.cancelEditAuxAccount', null)
                        return 'Select'
                    }
                    else {
                        return 'SubjectDisplay'
                    }
                }
            }
            return 'SubjectDisplay'
        }
    }

    getDisabledDate = (current) => {
        var disabledDate = new Date(this.metaAction.gf('data.other.disabledDate'))

        return current && current.valueOf() < disabledDate
    }

    isFocusOnAccount = (ctrlPath) => {
        let activeElement = document.getElementById('auxItemConfirm')
        let isFocus = this.metaAction.isFocus(ctrlPath)

        return isFocus && (activeElement == null || activeElement == undefined)

    }

    onFieldFocus = async (path) => {
        if (path.indexOf('root.children.details.columns.summary') > -1) {
            let rowIndex = parseInt(path.split(',')[1]),
                detail = this.metaAction.gf('data.form.details').get(rowIndex)

            this.backCaculateAmount()

            const summaryList = await this.webapi.summary.query()
            const templateList = await this.webapi.commonDoc.query()
            this.metaAction.sf('data.other.summarys', fromJS([...summaryList.summarys, ...this.translateTemplate(templateList)]))
            this.extendAction.gridAction.cellAutoFocus()
        } else if (path.indexOf('accountingSubject') > -1) {
            // 当凭证中的科目被停用后，再次点击科目时，显示科目名及辅助项目 START 0110
            let rowIndex = parseInt(path.split(',')[1]),
                detail = this.metaAction.gf('data.form.details').get(rowIndex),
                accountingSubject = detail.get('accountingSubject')

            if (!!accountingSubject) {
                let codeAndName = accountingSubject.get('codeAndName'),
                    accountId = accountingSubject.get('id'),
                    dom = document.getElementsByClassName('ant-select-selection-selected-value'),
                    accountingSubjects = this.metaAction.gf('data.other.accountingSubjects'),
                    itemData = accountingSubjects.find(o => o.get('id') == accountId),
                    auxAccountSubjects = accountingSubject.get('auxAccountSubjectsPreSelected')

                if (!itemData) {
                    if (dom && dom.length > 0) {
                        let subjectWidthAuxName = this.getSubjectWithAuxName(codeAndName, accountingSubject, auxAccountSubjects)

                        dom[0].innerHTML = subjectWidthAuxName
                        dom[0].title = subjectWidthAuxName
                    }
                }
            }
            // 当凭证中的科目被停用后，再次点击科目时，显示科目名及辅助项目 END 0110

            this.backCaculateAmount()
        }
        this.injections.reduce('onFieldFocus', path)

        // 实现借方金额、贷方金额的onBlur之用
        if (this.metaAction.gf('data.other.originalFieldPath') == undefined) {
            this.metaAction.sf('data.other.originalFieldPath', path)
        }

        //数量外币PopOver框打开时
        if (this.isVisibleQuanAndCurrCom()) {
            let btnQuantityCancels = document.getElementsByClassName('btnQuantityCancel')
            for (var i = btnQuantityCancels.length - 1; i > -1; i--) {
                let btnQuantityCancel = btnQuantityCancels[i],
                    popOver = btnQuantityCancels[i].parentElement.parentElement.parentElement.parentElement.parentElement.parentElement.parentElement

                if (popOver && popOver.className.indexOf('ant-popover-hidden') == -1 && btnQuantityCancel) {
                    btnQuantityCancel.click()
                }
            }
        }
    }

    getSubjectWithAuxName(subjectName, accountingSubject, auxAccountSubjects) {
        if (!accountingSubject.get('isCalc')) return subjectName

        let subjectWithAuxName = ''

        if (accountingSubject.get('isCalcCustomer') && auxAccountSubjects.get('customer') && auxAccountSubjects.get('customer').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('customer').get('name')
        }
        if (accountingSubject.get('isCalcSupplier') && auxAccountSubjects.get('supplier') && auxAccountSubjects.get('supplier').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('supplier').get('name')
        }
        if (accountingSubject.get('isCalcProject') && auxAccountSubjects.get('project') && auxAccountSubjects.get('project').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('project').get('name')
        }
        if (accountingSubject.get('isCalcDepartment') && auxAccountSubjects.get('department') && auxAccountSubjects.get('department').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('department').get('name')
        }
        if (accountingSubject.get('isCalcPerson') && auxAccountSubjects.get('person') && auxAccountSubjects.get('person').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('person').get('name')
        }
        if (accountingSubject.get('isCalcInventory') && auxAccountSubjects.get('inventory') && auxAccountSubjects.get('inventory').get('name')) {
            subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get('inventory').get('name')
        }

        //自定义档案
        let calcDict = this.metaAction.gf('data.other.calcDict'),
            userDefineItemName
        for (var j = 1; j <= 10; j++) {
            if (accountingSubject.get(`isExCalc${j}`) && auxAccountSubjects.get(`exCalc${j}`) && auxAccountSubjects.get(`exCalc${j}`).get('name')) {
                subjectWithAuxName = subjectWithAuxName + "_" + auxAccountSubjects.get(`exCalc${j}`).get('name')
            }
        }

        return subjectName + '_' + subjectWithAuxName
    }

    backCaculateAmount = () => {
        if (this.metaAction.gf('data.other.originalFieldPath')) {
            //处理某一分录行填写表格中的金额后，再点击该行数量外币列时，弹出框的金额没有被反算的问题 0310
            let originalFieldPath = this.metaAction.gf('data.other.originalFieldPath'),
                lastEditRowIndex = originalFieldPath.split(',')[1],
                detail = this.metaAction.gf('data.form.details').get(lastEditRowIndex)

            if(!detail) return

            let quantityAndForeignCurrency = detail.get('quantityAndForeignCurrency')

            if (!quantityAndForeignCurrency) return

            let amount = quantityAndForeignCurrency.get('amount'),
                lastFocusPath = this.metaAction.gf('data.other.originalFieldPath'),
                lastEditField

            if (lastFocusPath && lastFocusPath.indexOf('debitAmount') != -1) {
                lastEditField = 'debitAmount'
            } else if (lastFocusPath && lastFocusPath.indexOf('creditAmount') != -1) {
                lastEditField = 'creditAmount'
            } else {
                lastEditField = undefined
            }

            if (lastEditField && amount != detail.get(lastEditField)) {
                this.injections.reduce('onEvent', 'onBlur', { path: originalFieldPath })
            }
        }
    }

    //公共方法
    isVisibleQuanAndCurrCom = () => {
        let ret = false

        if (document.getElementsByClassName) {
            let accountQuantityEdit = document.getElementsByClassName('accountQuantityEdit')[0],
                quantityPopOver = document.getElementsByClassName('ant-popover')

            if (accountQuantityEdit) {
                if (quantityPopOver && quantityPopOver.length > 0) {
                    for (var i = 0; i < quantityPopOver.length; i++) {
                        if (quantityPopOver[i] && quantityPopOver[i].className.indexOf('ant-popover-hidden') == -1) {
                            ret = true
                            break
                        }
                    }
                }
            }
        }

        return ret
    }

    onEvent = async (eventName, option) => {
        if (!option.path) {
            return
        }

        if (eventName === 'accountQuantityIsShow') {
            this.injections.reduce('accountQuantityIsShow', option)
            this.extendAction.gridAction.cellAutoFocus()
        } else if (eventName === 'cancelEditAuxAccount') {
            this.injections.reduce('cancelEditAuxAccount', option)
            this.metaAction.focus(option.path)
            this.extendAction.gridAction.cellAutoFocus()
        } else if (eventName === 'endEditAuxAccount') {
            let curIndex = option.path.split(',')[1],
                details = this.metaAction.gf('data.form.details'),
                curAccountSubject = details.get(curIndex).get('accountingSubject'),
                editStatus = this.metaAction.gf('data.other.editStatus'),
                balance

            if (editStatus == data.ADD_STATUS || editStatus == data.EDIT_STATUS) {
                balance = await this.getAccountBalance(option.path, undefined, option)
                option.balance = balance
                option.initBalance = balance
            }

            this.injections.reduce('endEditAuxAccount', option, balance)
            this.extendAction.gridAction.cellAutoFocus()
        } else {
            this.injections.reduce('onEvent', eventName, option)
        }
    }

    //把服务端返回的字段名, 做转换
    translateTemplate = (templates) => {
        let templateList = List()
        for (let i = 0; i < templates.length; i++) {
            templateList = templateList.push({
                id: templates[i].docTemplateId,
                code: templates[i].docTemplateCode,
                name: templates[i].docTemplateName + ' [模板]',
                type: 'template'
            })
        }

        return templateList.toJS()
    }

    filterOption = (inputValue, option) => {
        if (option && option.props && option.props.value) {
            let accountingSubjects = this.metaAction.gf('data.other.accountingSubjects')
            let itemData = accountingSubjects.find(o => o.get('id') == option.props.value)

            if ((itemData.get('code') && itemData.get('code').indexOf(inputValue) == 0)
                || (itemData.get('gradeName') && itemData.get('gradeName').indexOf(inputValue) != -1)
                || (itemData.get('codeAndName') && itemData.get('codeAndName').indexOf(inputValue) == 0)
                || (itemData.get('helpCode') && itemData.get('helpCode').indexOf(inputValue.toUpperCase()) != -1)
                || (itemData.get('helpCodeFull') && itemData.get('helpCodeFull').indexOf(inputValue) != -1)) {

                //将滚动条置顶
                let select = document.getElementsByClassName('ant-select-dropdown-menu')
                if (select.length > 0 && select[0].scrollTop > 0) {
                    select[0].scrollTop = 0
                }

                return true
            }
            else {
                return false
            }
        }

        return true
    }

    filterOptionSummary = (input, option) => {
        if (option && option.props && option.props.children) {
            return option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0
        }

        return true
    }

    clearGridFocus = () => {
        this.metaAction.sf('data.other.focusFieldPath', undefined)
    }

    certificateManagement = () => {
        this.component.props.setPortalContent &&
            this.component.props.setPortalContent('凭证管理', 'app-proof-of-list')
    }

    getAudited = () => {
        return (this.metaAction.gf('data.form.certificateStatus') == consts.VOUCHERSTATUS_Approved)
    }

    isRowOperation = () => {
        return this.metaAction.gf('data.form.certificateStatus') == data.STATUS_VOUCHER_NOT_AUDITED
    }

    audit = async () => {
        let id = this.metaAction.gf('data.form.id'),
            ts = this.metaAction.gf('data.form.ts'),
            certificateStatus = this.metaAction.gf('data.form.certificateStatus')

        if (!id || !ts) {
            this.metaAction.toast('success', '请先保存凭证！')
            return
        }

        if (certificateStatus == data.STATUS_VOUCHER_NOT_AUDITED) {
            const response = await this.webapi.certificate.audit({ docId: id, ts: ts })
            if (response) {
                this.metaAction.toast('success', '凭证审核成功！')
                this.injections.reduce('changeAuditStatus', data.STATUS_VOUCHER_AUDITED, this.getCurrentUser(), response)
            }
        } else {
            const response = await this.webapi.certificate.antiAudit({ docId: id, ts: ts })
            if (response) {
                this.metaAction.toast('success', '凭证反审核成功！')
                this.injections.reduce('changeAuditStatus', data.STATUS_VOUCHER_NOT_AUDITED, this.getCurrentUser(), response)
            }
        }
    }

    commonMenuClick = (e) => {
        switch (e.key) {
            case 'useCommon':
                this.uesCommon()
                break;
            case 'saveCommon':
                this.saveCommon()
                break;
        }
    }

    uesCommon = () => {
        this.checkEditStatusBeforeLoadNewVoucher(async () => {
            const ret = await this.metaAction.modal('show', {
                title: '凭证模版',
                width: 880,
                height: 425,
                okText: '使用凭证模版',
                bodyStyle: {padding: '10px 50px'},
                children: this.metaAction.loadApp('ttk-voucher-app-document-common', {
                    store: this.component.props.store,
                    columnCode: "common"
                }),
            })

            if (ret) {
                let certificateTemplate = await this.webapi.commonDoc.findById({ docTemplateId: ret.docTemplateId })
                certificateTemplate.entrys = await this.getMultiAccountBalance(certificateTemplate.entrys)
                //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
                // this.component.props.editing(this.component.props.appName, true)
                this.editCloseTips(true)
                this.injections.reduce('applyCertificateTemplate', fromJS(certificateTemplate), data.ADD_STATUS)
            }
        })
    }
    editCloseTips=(istip)=>{
        if (this.component.props.editing){
            //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
            this.component.props.editing(this.component.props.appName, istip)
        }
    }
    saveCommon = async () => {
        if (!this.canSaveAsTemplate()) {
            this.metaAction.toast('warning', '没有填写科目的凭证不能存为模板！')
            return
        }

        const ret = await this.metaAction.modal('show', {
            title: '存为凭证模版',
            width: 340,
            children: this.metaAction.loadApp('ttk-voucher-app-document-common-add', {
                store: this.component.props.store,
                columnCode: "common",
                initData: { certificateData: this.metaAction.gf('data.form') }
            }),
        })
    }

    //能否存为模板(空凭证不能存为模板)
    canSaveAsTemplate = () => {
        let details = this.metaAction.gf('data.form.details')

        for (let item of details) {
            let itemForServer = this.convertVoucherItemForServer(item)
            if (itemForServer) {
                return true
            }
        }
        return false
    }

    getAccountBalance = async (path, accountingSubject, option) => {
        let balance

        if (this.metaAction.gf('data.other.editStatus') == data.ADD_STATUS) {
            let index = path.split(',')[1],
                subjectValue = this.metaAction.gf('data.form.details').get(index).get('accountingSubject')

            let queryParams = {},
                auxAccountSubjects

            if (option) {
                auxAccountSubjects = option.data.get('auxAccountSubjects')
                queryParams.accountId = subjectValue.get('id')
            } else {
                queryParams.accountId = accountingSubject.get('id')
            }

            if (!!auxAccountSubjects) {
                queryParams.departmentId = auxAccountSubjects.get('department') ? auxAccountSubjects.get('department').get('id') : undefined
                queryParams.personId = auxAccountSubjects.get('person') ? auxAccountSubjects.get('person').get('id') : undefined
                queryParams.customerId = auxAccountSubjects.get('customer') ? auxAccountSubjects.get('customer').get('id') : undefined
                queryParams.supplierId = auxAccountSubjects.get('supplier') ? auxAccountSubjects.get('supplier').get('id') : undefined
                queryParams.inventoryId = auxAccountSubjects.get('inventory') ? auxAccountSubjects.get('inventory').get('id') : undefined
                queryParams.projectId = auxAccountSubjects.get('project') ? auxAccountSubjects.get('project').get('id') : undefined

                for (var j = 1; j <= 10; j++) {
                    queryParams[`exCalc${j}`] = auxAccountSubjects.get(`exCalc${j}`) ? auxAccountSubjects.get(`exCalc${j}`).get('id') : undefined
                }
            }

            const response = await this.webapi.certificate.getAccountBalance(queryParams)
            if (response != undefined) {
                balance = response
            }
        } else {
            balance = 0
        }

        return balance
    }

    // 获取多个科目的余额
    getMultiAccountBalance = async (entrys) => {
        let balance

        if (this.metaAction.gf('data.other.editStatus') == data.ADD_STATUS) {
            let queryParams = [], param, entry

            for (var i = 0; i < entrys.length; i++) {
                entry = entrys[i]
                param = {}
                param.accountId = entry.accountId
                param.departmentId = entry.departmentId ? entry.departmentId : undefined
                param.personId = entry.personId ? entry.personId : undefined
                param.customerId = entry.customerIdcustomerId ? entry.customerId : undefined
                param.supplierId = entry.supplierId ? entry.supplierId : undefined
                param.inventoryId = entry.inventoryId ? entry.inventoryId : undefined
                param.projectId = entry.projectId ? entry.projectId : undefined

                for (var j = 1; j <= 10; j++) {
                    param[`exCalc${j}`] = entry[`exCalc${j}`] ? entry[`exCalc${j}`] : undefined
                }

                queryParams.push(param)
            }

            const response = await this.webapi.certificate.getAccountsBalance(queryParams)
            if (response != undefined) {
                for (var i = 0; i < entrys.length; i++) {
                    entrys[i].balance = response[i].balance
                }
            }
        } else {
            entrys
        }

        return entrys
    }

    print = async () => {
        let _this = this

        const {
            height,
            maxLineNum,
            printAuxAccCalc,
            type,
            width
        } = await this.webapi.certificate.getPrintConfig()

        this.metaAction.modal('show', {
            title: '打印',
            width: 400,
            footer: null,
            iconType: null,
            okText: '打印',
            className: 'print-modal-container',
            children: <PrintOption
                          height={height}
                          maxLineNum={maxLineNum}
                          printAuxAccCalc={printAuxAccCalc}
                          type={type}
                          width={width}
                          callBack={ _this.submitPrintOption }
                      />
        })
    }

    submitPrintOption = async (form, target) => {
        let id = this.metaAction.gf('data.form.id')
        let option = { "type": parseInt(form.state.value), "printAuxAccCalc": form.state.printAccountChecked, "docIdsStr": `${id}` }
        if (form.state.value == "0") {
            Object.assign(option, { "maxLineNum": form.state.pageSize })
        } else if (form.state.value == "2") {
            Object.assign(option, { "height": form.state.height, "width": form.state.width })
        }
        let res = await this.webapi.certificate.print(option)
    }

    checkVoucherDataBeforeSave(form) {
        let msg = [], allItemEmpty = true, isBackCaculate = false,
            details = this.metaAction.gf('data.form.details')

        this.backCaculateAmount()

        //凭证号校验
        if (!form.code) {
            msg.push('凭证号不能为空')
        } else if (!/^\d+$/.test(form.code)) {
            msg.push('凭证号必须为纯数字，请修改')
        }

        if (form.code === '0') msg.push('凭证号不能为0，请修改')
        if (!form.date) msg.push('日期不能为空')

        form.details.forEach((detail, i) => {
            if (!detail ||
                (!detail.accountingSubject && !detail.summary && !detail.creditAmount && !detail.debitAmount) ||
                (!detail.accountingSubject && detail.summary && !detail.creditAmount && !detail.debitAmount)) {
                return
            }

            allItemEmpty = false

            //凭证号校验
            //2) 摘要、科目和金额 同时填写或不填
            let emptyItemNames = List()

            if (!detail.summary) {
                emptyItemNames = emptyItemNames.push('摘要')
            }

            if (!detail.accountingSubject) {
                emptyItemNames = emptyItemNames.push('科目')
            }

            if (!detail.debitAmount && !detail.creditAmount) {
                emptyItemNames = emptyItemNames.push('金额')
            }

            if (emptyItemNames.size > 0) {
                let rowError
                if (emptyItemNames.size > 2) {
                    rowError = "摘要、科目和金额"
                } else if (emptyItemNames.size == 2) {
                    rowError = emptyItemNames.get(0) + '和' + emptyItemNames.get(1)
                } else {
                    rowError = emptyItemNames.get(0)
                }
                msg.push("第" + (i + 1) + "行，请填写" + rowError)
                return
            }

            if (detail.summary && detail.summary.length > 200)
                msg.push("第" + (i + 1) + "行，摘要输入过长，请调整")

            //3) 数量/外币 核算
            if (detail.accountingSubject.isCalcQuantity
                && (!detail.quantityAndForeignCurrency || !detail.quantityAndForeignCurrency.amount)) {

                let lastFocusPath = this.metaAction.gf('data.other.originalFieldPath'),
                    rowIndex = lastFocusPath ? lastFocusPath.split(',')[1] : -1,
                    lastEditField

                // 解决360、IE、Edge浏览器下保存时借贷方金额未反算的问题 TTK-3055
                if (lastFocusPath && rowIndex == i) {
                    if (lastFocusPath && lastFocusPath.indexOf('debitAmount') != -1) {
                        lastEditField = 'debitAmount'
                    } else if (lastFocusPath && lastFocusPath.indexOf('creditAmount') != -1) {
                        lastEditField = 'creditAmount'
                    } else {
                        lastEditField = undefined
                    }

                    details = details.update(rowIndex, item=>item.setIn(['quantityAndForeignCurrency', 'amount'], detail[lastEditField]))
                    isBackCaculate = true
                } else {
                    msg.push("第" + (i + 1) + "行，请填写数量核算")
                    return
                }

            }
            if (detail.accountingSubject.isCalcMulti
                && (!detail.quantityAndForeignCurrency || !detail.quantityAndForeignCurrency.currency)) {
                msg.push("第" + (i + 1) + "行，请填写外币核算")
                return
            }
            //4) 启用辅助核算
            if (detail.accountingSubject.isCalc) {
                if (!detail.accountingSubject.auxAccountSubjects) {
                    msg.push("第" + (i + 1) + "行，请填写科目辅助核算")
                    return
                }
                let auxAccountSubjects = detail.accountingSubject.auxAccountSubjects

                //部门
                if (detail.accountingSubject.isCalcDepartment
                    && !(auxAccountSubjects.department && auxAccountSubjects.department.id)) {
                    msg.push("第" + (i + 1) + "行，请填写部门辅助核算")
                    return
                }

                //人员
                if (detail.accountingSubject.isCalcPerson
                    && !(auxAccountSubjects.person && auxAccountSubjects.person.id)) {
                    msg.push("第" + (i + 1) + "行，请填写人员辅助核算")
                    return
                }

                //客户
                if (detail.accountingSubject.isCalcCustomer
                    && !(auxAccountSubjects.customer && auxAccountSubjects.customer.id)) {
                    msg.push("第" + (i + 1) + "行，请填写客户辅助核算")
                    return
                }

                //供应商
                if (detail.accountingSubject.isCalcSupplier
                    && !(auxAccountSubjects.supplier && auxAccountSubjects.supplier.id)) {
                    msg.push("第" + (i + 1) + "行，请填写供应商辅助核算")
                    return
                }

                //存货
                if (detail.accountingSubject.isCalcInventory
                    && !(auxAccountSubjects.inventory && auxAccountSubjects.inventory.id)) {
                    msg.push("第" + (i + 1) + "行，请填写存货辅助核算")
                    return
                }

                //项目
                if (detail.accountingSubject.isCalcProject
                    && !(auxAccountSubjects.project && auxAccountSubjects.project.id)) {
                    msg.push("第" + (i + 1) + "行，请填写项目辅助核算")
                    return
                }

                //自定义档案
                let calcDict = this.metaAction.gf('data.other.calcDict')
                for (var j = 1; j <= 10; j++) {
                    if (detail.accountingSubject[`isExCalc${j}`]
                        && !(auxAccountSubjects[`exCalc${j}`] && auxAccountSubjects[`exCalc${j}`].id)) {

                        let userDefineItemName = calcDict.get(`isExCalc${j}`)
                        msg.push("第" + (i + 1) + `行，请填写${userDefineItemName}辅助核算`)
                        return
                    }
                }
            }
        })

        if (isBackCaculate){
            this.metaAction.sf('data.form.details', details)
        }

        //5) 不能全为空行
        if (allItemEmpty) {
            msg.push("请填写凭证分录")
        } else {
            //6) 借贷平衡
            let debitSum = 0,
                creditSum = 0
            for (let item of form.details) {
                let debitAmountItem = parseInt((parseFloat(item.debitAmount || 0) * 100).toFixed(0)),
                    creditAmountItem = parseInt((parseFloat(item.creditAmount || 0) * 100).toFixed(0))
                debitSum = (parseInt((debitSum * 100).toFixed(0)) + debitAmountItem) / 100
                creditSum = (parseInt((creditSum * 100).toFixed(0)) + creditAmountItem) / 100
            }
            if (debitSum.toFixed(2) !== creditSum.toFixed(2)) {
                msg.push("凭证借贷不平")
            }
        }
        return msg
    }

    save = async (callback) => {
        let form = this.metaAction.gf('data.form').toJS()
        const msg = this.checkVoucherDataBeforeSave(form),
            checkStyle = { textAlign: 'left', fontSize: '14px', display: 'inline-block', verticalAlign: 'top' }

        if (msg.length > 0) {
            this.metaAction.toast('warning',
                <div style={checkStyle}>
                    {msg.map((o, index) => <p style={{ marginBottom: '0' }}>{(index+1) + '.' + o}</p>)}
                </div>
            )
            return
        }

        let certificateData = this.getCertificateData()

        //让保存按钮置灰，防止并发 置灰
        this.injections.reduce('showBtnState', {path: 'data.other.btnStatus',value: true})

        //新增
        if (this.metaAction.gf('data.other.editStatus') == data.ADD_STATUS || !certificateData.docId) {

            const response = await this.webapi.certificate.create(certificateData)
            if (response) {
                this.metaAction.toast('success', '凭证保存成功')
                this.injections.reduce('loadCertificate', fromJS(response), data.VIEW_STATUS)
                if (callback && typeof callback == 'function') callback()
                //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
                // this.component.props.editing(this.component.props.appName, false)
                this.editCloseTips(false)
            }
        }
        //更新
        else {
            const response = await this.webapi.certificate.update(certificateData)
            if (response) {
                this.metaAction.toast('success', '凭证更新成功')
                this.injections.reduce('loadCertificate', fromJS(response), data.VIEW_STATUS)
                if (callback && typeof callback == 'function') callback()
                //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
                // this.component.props.editing(this.component.props.appName, false)
                this.editCloseTips(false)
            }
        }

        //让保存按钮置灰，防止并发 重置
        this.injections.reduce('showBtnState', { path: 'data.other.btnStatus', value: false })
    }

    saveAndNew = async () => {
        let form = this.metaAction.gf('data.form').toJS()
        let errorMsg = this.checkVoucherDataBeforeSave(form),
            checkStyle = { textAlign: 'left', fontSize: '14px', display: 'inline-block', verticalAlign: 'top' }

        if (errorMsg.length > 0) {
            this.metaAction.toast('warning',
                <div style={checkStyle}>
                    {errorMsg.map((o, index) => <p style={{ marginBottom: '0' }}>{(index+1) + '.' + o}</p>)}
                </div>
            )
            return
        }

        let certificateData = this.getCertificateData()

        //让保存按钮置灰，防止并发 置灰
        this.injections.reduce('showBtnState', {
            path: 'data.other.btnStatus',
            value: true
        })

        //新增
        if (this.metaAction.gf('data.other.editStatus') == data.ADD_STATUS || !certificateData.docId) {
            const response = await this.webapi.certificate.create(certificateData)
            if (response) {
                this.metaAction.toast('success', '凭证保存成功')

                let certificateMoment = moment(certificateData.voucherDate),
                    year = certificateMoment.year(),
                    period = certificateMoment.month() + 1,
                    option = {year: year, period: period, isReturnValue: true}

                let initCertificateData = await this.webapi.certificate.init(option)
                //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
                // this.component.props.editing(this.component.props.appName, false)
                this.editCloseTips(false)
                if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
                    this.metaAction.toast('error', initCertificateData.error.message)
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: certificateData.voucherDate })
                } else {
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: certificateData.voucherDate })
                    this.extendAction.gridAction.cellAutoFocus()
                }
            }
        }
        //更新
        else {
            const response = await this.webapi.certificate.update(certificateData)
            if (response) {
                this.metaAction.toast('success', '凭证更新成功')
                let certificateMoment = moment(certificateData.voucherDate),
                    year = certificateMoment.year(),
                    period = certificateMoment.month() + 1,
                    option = {year: year, period: period, isReturnValue: true}

                let initCertificateData = await this.webapi.certificate.init(option)
                //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
                // this.component.props.editing(this.component.props.appName, false)
                this.editCloseTips(false)
                if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
                    this.metaAction.toast('error', initCertificateData.error.message)
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: certificateData.voucherDate })
                } else {
                    this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: certificateData.voucherDate })
                    this.extendAction.gridAction.cellAutoFocus()
                }
            }
        }

        //让保存按钮置灰，防止并发 重置
        this.injections.reduce('showBtnState', {
            path: 'data.other.btnStatus',
            value: false
        })
    }

    //组装当前界面的数据,供新增接口
    getCertificateData = () => {
        let certificate = {},
            form = this.metaAction.gf('data.form')

        certificate.docType = form.get('docType')
        certificate.docCode = form.get('code')
        certificate.enclosures = this.getEnclosures(form.get('attachmentFiles').toJS())

        let certificateMoment = moment(form.get('date'))

        certificate.voucherDate = certificateMoment.format('YYYY-MM-DD')
        certificate.year = certificateMoment.year()
        certificate.period = certificateMoment.month() + 1
        certificate.attachedNum = form.get('attachCount')
        //是否修改过凭证号
        certificate.isModifiedDocCode = (certificate.docCode != this.metaAction.gf('data.form.initVoucherCode'))
        //是否为插入凭证
        certificate.isInsert = this.metaAction.gf('data.other.isInsert')

        //凭证号小于4为自动补0
        certificate.docCode = certificate.docCode.padStart(4, '0')

        //考虑修改的情况: 单据id
        let oldId = form.get('id')
        if (oldId) {
            certificate.docId = oldId
            certificate.ts = form.get('ts')
        }

        let entrys = [],
            details = form.get('details')
        for (let item of details) {
            let entry = this.convertVoucherItemForServer(item, true)

            if (entry) {
                entrys.push(entry)
            }
        }

        //考虑修改的情况: 删除分录
        let deletedCertificateItems = this.metaAction.gf('data.other.deletedCertificateItems')
        if (deletedCertificateItems && deletedCertificateItems.size > 0) {
            for (let delItem of deletedCertificateItems) {
                entrys.push({ id: delItem.get('id'), rowStatus: 3, ts: delItem.get('ts') })
            }
        }

        certificate.entrys = entrys

        return certificate
    }

    getEnclosures = (attachmentFiles) => {
        let enclosures = []
        let fileList = attachmentFiles || []

        if (fileList && fileList.length > 0) {
            fileList.map(o => {
                enclosures.push({
                    'id': o.id,
                    'file': { 'id': o.fileId, 'type': o.type,'alt': o.alt, 'name': o.name, 'ts': o.ts, 'accessUrl': o.accessUrl},
                    'operateStatus': 1
                })
                return enclosures
            })
        }
        this.injections.reduce('upload', enclosures, undefined, fileList ? fileList.length : undefined)
        return enclosures
    }

    convertVoucherItemForServer = (item, isForUpdate) => {
        if (!item || !item.get('accountingSubject') || (!item.get('debitAmount') && !item.get('creditAmount'))) {
            return undefined
        }

        //如果需要数量/外币 核算,但是没有填写
        if ((item.get('accountingSubject').get('isCalcQuantity') || item.get('accountingSubject').get('isCalcMulti')) &&
            !item.get('quantityAndForeignCurrency')) {
            return undefined
        }

        //如果需要辅助核算,但是没有填写
        if (item.get('accountingSubject').get('isCalc') && !item.get('accountingSubject').get('auxAccountSubjects')) {
            return undefined
        }

        let entry = {
            id: item.get('id') ? item.get('id') : null,
            summary: item.get('summary'),
            accountId: item.get('accountingSubject').get('id'),
            origAmountDr: item.get('debitAmount'),
            amountDr: item.get('debitAmount'),
            origAmountCr: item.get('creditAmount'),
            amountCr: item.get('creditAmount'),
            ts: item.get('ts') ? item.get('ts') : null,
            inPutTaxDeductId: item.get('inPutTaxDeductId') ? item.get('inPutTaxDeductId') : null
        }

        //考虑修改的情况: 修改分录
        if (isForUpdate) {
            if (item.get('id')) {
                entry.id = item.get('id')
                entry.rowStatus = 2
                entry.ts = item.get('ts')
            }
            else {
                entry.rowStatus = 1  //--分录状态 0:未变化  1:新增  2:修改  3:删除
            }
        }

        //数量辅助核算
        if (item.get('accountingSubject').get('isCalcQuantity')) {
            entry.quantity = item.get('quantityAndForeignCurrency').get('quantity')
            entry.price = item.get('quantityAndForeignCurrency').get('price')
            entry.unitId = item.get('accountingSubject').get('unitId') //计量单位, 暂不支持编辑, 所以用科目里设置的即可
            entry.exchangeRate = 1
        } else {
            entry.quantity = null
            entry.price = null
            entry.unitId = null
            entry.exchangeRate = null
        }

        //外币辅助核算
        if (item.get('accountingSubject').get('isCalcMulti')) {
            entry.currencyId = item.get('quantityAndForeignCurrency').get('currency') ? item.get('quantityAndForeignCurrency').get('currency').get('id') : ''
            entry.exchangeRate = item.get('quantityAndForeignCurrency').get('exchangeRate')
            if (entry.amountDr) {
                entry.origAmountDr = item.get('quantityAndForeignCurrency').get('origAmount') || 0
            }
            else {
                entry.origAmountCr = item.get('quantityAndForeignCurrency').get('origAmount') || 0
            }
        } else {
            entry.currencyId = null
            entry.exchangeRate = 1
            entry.origAmountDr = null
            entry.origAmountCr = null
        }

        //其他辅助核算
        if (item.get('accountingSubject').get('isCalc')) {
            let auxAccountSubjects = item.get('accountingSubject').get('auxAccountSubjects')
            //部门
            if (item.get('accountingSubject').get('isCalcDepartment')) {
                entry.departmentId = auxAccountSubjects.get('department').get('id')
            } else {
                entry.departmentId = null //为了保持上下分录属性字段数量一致，需传null 临时方案 haozhao
            }

            //人员
            if (item.get('accountingSubject').get('isCalcPerson')) {
                entry.personId = auxAccountSubjects.get('person').get('id')
            } else {
                entry.personId = null
            }

            //客户
            if (item.get('accountingSubject').get('isCalcCustomer')) {
                entry.customerId = auxAccountSubjects.get('customer').get('id')
            } else {
                entry.customerId = null
            }

            //供应商
            if (item.get('accountingSubject').get('isCalcSupplier')) {
                entry.supplierId = auxAccountSubjects.get('supplier').get('id')
            } else {
                entry.supplierId = null
            }

            //存货
            if (item.get('accountingSubject').get('isCalcInventory')) {
                entry.inventoryId = auxAccountSubjects.get('inventory').get('id')
            } else {
                entry.inventoryId = null
            }

            //项目
            if (item.get('accountingSubject').get('isCalcProject')) {
                entry.projectId = auxAccountSubjects.get('project').get('id')
            } else {
                entry.projectId = null
            }

            //自定义档案
            for (var j = 1; j <= 10; j++) {
                if (item.get('accountingSubject').get(`isExCalc${j}`)) {
                    entry[`exCalc${j}`] = auxAccountSubjects.get(`exCalc${j}`).get('id')
                } else {
                    entry[`exCalc${j}`] = null
                }
            }
        } else {
            entry.customerId = null
            entry.departmentId = null
            entry.personId = null
            entry.inventoryId = null
            entry.supplierId = null
            entry.projectId = null
            for (var j = 1; j <= 10; j++) {
                entry[`exCalc${j}`] = null
            }
        }
        return entry
    }

    onSummarySelect = (path, rowIndex, row, src) => async (v) => {
        const hit = src.find(o => o.name == v)
        if (hit) {
            if (hit.type == 'template') {
                if (!!v) {
                    v = v.replace(' [模板]', '')
                }

                let response = await this.webapi.commonDoc.findById({ docTemplateName: v })
                response.entrys = await this.getMultiAccountBalance(response.entrys)
                this.injections.reduce('applySummaryTemplate', fromJS(response), rowIndex)
            } else {
                this.metaAction.sf('data.form.details.' + rowIndex + '.summary', hit.name)
                this.injections.reduce('onEvent', 'onEndEdit', { path: path, data: fromJS(hit), isChanged: true })
                this.extendAction.gridAction.cellAutoFocus()
            }
        }
        else {
            this.metaAction.sf('data.form.details.' + rowIndex + '.summary', v)
            let edit_status = this.metaAction.gf('data.other.editStatus')
            if (edit_status != data.ADD_STATUS) {
                this.metaAction.sf('data.other.editStatus', data.EDIT_STATUS)
            }
        }
        //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
        // this.component.props.editing(this.component.props.appName, true)
        this.editCloseTips(true)
    }

    onSummaryChange = (rowIndex) => async (v) => {
        this.metaAction.sf('data.form.details.' + rowIndex + '.summary', v)
        let edit_status = this.metaAction.gf('data.other.editStatus')
        if (edit_status != data.ADD_STATUS) {
            this.metaAction.sf('data.other.editStatus', data.EDIT_STATUS)
        }
    }

    onFieldChange = async (path, oldValue, newValue) => {
        if (path.indexOf('code') == -1 &&
            this.metaAction.gf('data.other.editStatus') == data.VIEW_STATUS &&
            this.metaAction.gf('data.form.certificateStatus') != data.STATUS_VOUCHER_AUDITED &&
            oldValue != newValue) {

            this.injections.reduce('changeStatus', data.EDIT_STATUS, data.STATUS_VOUCHER_NOT_AUDITED)
        } else if ((path.indexOf('code') > -1) &&
            this.metaAction.gf('data.other.editStatus') == data.VIEW_STATUS &&
            this.metaAction.gf('data.form.certificateStatus') != data.STATUS_VOUCHER_AUDITED) {

            this.injections.reduce('changeStatus', data.EDIT_STATUS, data.STATUS_VOUCHER_NOT_AUDITED)
        }

        //修改日期: 联动凭证号
        if (path.indexOf('date') > -1) {
            let oldDate = moment(oldValue),
                newDate = moment(newValue)

            if (oldDate.year() != newDate.year() || oldDate.month() != newDate.month()) {
                let initCertificateData = await this.webapi.certificate.getNewDocCode({ year: newDate.year(), period: newDate.month() + 1 })
                this.injections.reduce('initVoucherCode', initCertificateData.docCode)
            }
        }
        //附件数,上限限制为1W
        else if (path.indexOf('attachment') > -1) {
            if (isNaN(newValue) || parseInt(newValue) > data.MAX_ATTACH_COUNT) {
                this.metaAction.toast('warning', '附件数最大为' + data.MAX_ATTACH_COUNT)
            } else {
                if (newValue != oldValue) {
                    this.metaAction.sf('data.form.attachCount', parseInt(newValue))
                    this.injections.reduce('changeStatus', data.EDIT_STATUS, data.STATUS_VOUCHER_NOT_AUDITED)
                }
            }
        }
        this.injections.reduce('onFieldChange', path, oldValue, newValue)
    }

    dateKeydown = (e) => {
        if(e.keyCode == 13){
            document.getElementsByClassName('ttk-voucher-app-document-form-header-attachment-number')[0].focus()
            document.getElementsByClassName('ant-calendar-picker-input')[0].removeEventListener('keydown', this.dateKeydown, false)
        }
    }

    onPressEnter = (fieldName) => {
        if (fieldName == 'code') {
            this.metaAction.sf('data.other.focusFieldPath', 'root.children.formHeader.children.date.children.date')
            document.getElementsByClassName('ant-calendar-picker-input')[0].focus()
            document.getElementsByClassName('ant-calendar-picker-input')[0].addEventListener('keydown', this.dateKeydown, false)
        } else if (fieldName == 'date') {
            this.metaAction.sf('data.other.focusFieldPath', 'root.children.formHeader.children.attachment.children.attachment')
        } else if (fieldName == 'attachment') {
            this.metaAction.sf('data.other.focusFieldPath', 'root.children.details.columns.summary.cell.cell,0')
            this.extendAction.gridAction.cellAutoFocus()
        }
    }

    onSubjectSelect = (path, rowIndex, accountingSubjects) => async (v) => {
        let details = this.metaAction.gf('data.form.details'),
            curAccountingSubject = details.get(rowIndex).get('accountingSubject'),
            editStatus = this.metaAction.gf('data.other.editStatus')

        const hit = accountingSubjects.find(o => o.id == v)
        let balance

        if (curAccountingSubject && curAccountingSubject.get('id') == hit.id) {
            if (!curAccountingSubject.get('isCalc') && (editStatus == data.ADD_STATUS || editStatus == data.EDIT_STATUS)) {
                balance = await this.getAccountBalance(path, fromJS(curAccountingSubject))
                curAccountingSubject.balance = this.getPageBalance(curAccountingSubject.code, rowIndex, balance)
                curAccountingSubject.initBalance = balance
            }
            this.injections.reduce('onEvent', 'onEndEdit', { path: path, data: fromJS(curAccountingSubject), isChanged: false })
        } else {
            if (!hit.isCalc && (editStatus == data.ADD_STATUS || editStatus == data.EDIT_STATUS)) {
                balance = await this.getAccountBalance(path, fromJS(hit))
                hit.balance = this.getPageBalance(hit.code, rowIndex, balance)
                hit.initBalance = balance
            }
            this.injections.reduce('onEvent', 'onEndEdit', { path: path, data: fromJS(hit), isChanged: true })
        }
        //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
        //this.component.props.editing(this.component.props.appName, true)
        this.editCloseTips(true)
        this.extendAction.gridAction.cellAutoFocus()
    }

    getPageBalance = (accountCode, rowIndex, initBalance) => {
        let details = this.metaAction.gf('data.form.details'),
            balance = initBalance

        for (var i = 0; i < details.size; i++) {
            if (i != rowIndex &&
                details.get(i).get('accountingSubject') &&
                details.get(i).get('accountingSubject').get('code') == accountCode) {

                balance = details.get(i).get('accountingSubject').get('balance')
            }
        }

        return balance
    }

    amountChange = (path) => (newValue, oldValue) => {
        //设置凭证当前单据状态，供单据页签关闭用 false：关闭不提醒 true 关闭提醒
        // this.component.props.editing(this.component.props.appName, true)
        this.editCloseTips(true)
        let editStatus = this.metaAction.gf('data.other.editStatus')
        if (newValue == '0') {
            if (oldValue != newValue) {  //点选右边修改科目使其保存可用
                if (editStatus == data.EDIT_STATUS || editStatus == data.VIEW_STATUS) {
                    this.injections.reduce('changeStatus', data.EDIT_STATUS, data.STATUS_VOUCHER_NOT_AUDITED)
                }
            }
            return
        }
        this.injections.reduce('onFieldChange', path, oldValue, newValue)
    }

    amountBlur = (path) => () => {
        let curPath = this.metaAction.gf('data.other.originalFieldPath')
        this.injections.reduce('onEvent', 'onBlur', { path: curPath })
        this.metaAction.sf('data.other.originalFieldPath', path)
    }

    onShortcutKey = (option) => {
        this.injections.reduce('onEvent', 'onShortcutKey', { path: option, keyEvent: event })
        if (event && (event.key == ' ' || event.keyCode == 32)) {
  				this.extendAction.gridAction.cellAutoFocus()
  			}
    }

    sum = (details, fieldName) => {
        return this.sumInternal(details, (a, b) => a + parseFloat(b[fieldName]))
    }

    getTotal = (details) => {
        let sumValue = this.sum(details, 'debitAmount')

        return '合计：' + utils.number.moneySmalltoBig(parseFloat(sumValue.toFixed(2)))
    }

    sumInternal(details, fn) {
        if (!details || details.length == 0)
            return '0'

        return details.reduce((a, b) => {
            let r = fn(a, b)
            return isNaN(r) ? a : r
        }, 0)
    }

    addSubject = rowIndex => async () => {
        const ret = await this.metaAction.modal('show', {
            title: '新增科目',
            width: 400,
            okText: '保存',
            children: this.metaAction.loadApp('ttk-voucher-app-document-subjects-add', {
                store: this.component.props.store,
                columnCode: "subjects",
                active: 'certificate'
            })
        })
        if (ret) {
            const subjectList = await this.getSubjectList()
            this.injections.reduce('setAccoutingSubject', rowIndex, subjectList, ret)

            let path = `root.children.details.columns.accountingSubject.cell.cell,${rowIndex}`

            this.onSubjectChange(path, rowIndex, subjectList.glAccounts)(ret.id)
        }
    }

    getAuditText = () => {
        const certificateStatus = this.metaAction.gf('data.form.certificateStatus')
        if (certificateStatus === data.STATUS_VOUCHER_AUDITED) {
            return '反审核'
        } else {
            return '审核'
        }
    }

    cellStyle = (ctrlPath) => {
        let isFocus = this.metaAction.isFocus(ctrlPath)
        if (isFocus) {
            return " editable-cell"
        }
    }

    //附件上传状态改变
    attachmentChange = async (info) => {
        this.injections.reduce('attachmentLoading', true)
		//mock 上传数据
	    this.upload({
		    "id": 4287748629779456,
		    "originalName": info.file.name,
		    "name": info.file.name,
		    "root": "ttk-dev",
		    "path": "ATTACHMENT/4281915938039808/",
		    "uploadUserId": 4281915936466944,
		    "type": 1000010006,
		    "size": 56054,
		    "suffix": info.file.type,
		    "md5": "opgfW8T6QWKfvizAeRjarA==",
		    "ts": "2018-03-27 16:47:37.0",
		    "accessUrl": "http://ttk-dev.oss-cn-beijing.aliyuncs.com/ATTACHMENT/4281915938039808/4281915936466944-4287748629779456-eae2fed153820be2d2e5af633154d4dd.jpeg?Expires=1522144057&OSSAccessKeyId=LTAIa9iudLuMqc12&Signature=LtEdUl+VBXSrG6H0zbSfwAubd2E=",
		    "uploadTime": "2018-03-27 16:47:37"
	    },"2018-03-27 16:47:37.0")
        // if (info.file.status === 'done') {
        //     if (info.file.response.error && info.file.response.error.message) {
        //         return
        //     } else if (info.file.response.result && info.file.response.value) {
        //         this.upload(info.file.response.value, info.file.response.value.ts)
        //     }
        // }
    }

	/**
	 * 附件上传成功事件处理
	 * @param  {[type]} imgName [description]
	 * @return {[type]}         [description]
	 */
    upload = async (file, ts) => {
        let form = this.metaAction.gf('data.form').toJS()
        let fileList = this.metaAction.gf('data.form.attachmentFiles').toJS()||[]
        let fileData = []
        fileList.push({
            "accessUrl": file.accessUrl,
            "ts": file.ts || '',
            "type": file.type,
            "name": file.originalName,
            "fileId": file.id
        })
        if(fileList.length > 20){
            this.metaAction.toast('warning', '最多上传20个附件')
            this.injections.reduce('attachmentLoading', false)
            return
        }

        if(form.id){//凭证已经存在
            let enclosures = []
            enclosures.push({
                "fileId": file.id,
                "operateStatus": 1,
                "file": {
                    "id": file.id,
                    "type": file.type
                }
            })
            let fileData = { "docId": form.id, enclosures: enclosures }
            let data = await this.webapi.certificate.updateEnclosure(fileData)
            fileList = []
            if(data){
                let newenclosures = data.enclosures
                newenclosures.map((o) => {
                    fileList.push({
                        "id": o.id,
                        "accessUrl": o.file.accessUrl,
                        "ts":o.ts || '',
                        "type": o.file.type,
                        "name": o.file.originalName,
                        "fileId": o.file.id
                    })
                })
                this.injections.reduce('upload', newenclosures, data.ts, fileList.length)
            }

        }else{
            this.injections.reduce('upload', fileList, ts, fileList.length)
        }

        this.injections.reduce('attachmentLoading', false)

    }

    getAccessToken = () => {
        let token = fetch.getAccessToken()

        return { token: token }
    }

    //附件的下载操作
    download = (ps) => {
        let alink = document.createElement('a')
        alink.download = ps.accessUrl
        alink.href = ps.accessUrl
        alink.click()
    }

    moreMenuClick = (e) => {
        switch (e.key) {
            case 'del':
                this.del()
                break
            case 'insert':
                this.insert()
                break
        }
    }

    delFile = async (index) => {
        this.injections.reduce('attachmentVisible', true)
        const ret = await this.metaAction.modal('confirm', {
            title: '删除',
            content: '确认删除?'
        })

        if (ret) {
            let form = this.metaAction.gf('data.form').toJS()
            let res = {
                "docId": form.id,
                "enclosures": [{
                    "id": index.id,
                    "operateStatus": 3,
                    "ts": index.ts,
                    "file": {
                        "id": index.fileId,
                        "type": index.type
                    }
                }]
            }
            // delete index
            form.attachmentFiles.map((o, i) => {
                if (o.fileId == index.fileId) {
                    form.attachmentFiles.splice(i, 1)
                }
                return form.attachmentFiles
            })

            if (form.id) {
                let data = await this.webapi.certificate.updateEnclosure(res)//
                if (data) {
                    console.log('data.attachedNum:' + data.attachedNum)
                    this.injections.reduce('upload', form.attachmentFiles, data.ts, data.attachedNum)
                    this.metaAction.toast('success', '删除成功！')
                }
            }else{
                this.injections.reduce('upload', form.attachmentFiles, index.ts, form.attachmentFiles.length)
            }

        }
    }

    del = async () => {
        let id = this.metaAction.gf('data.form.id'),
            ts = this.metaAction.gf('data.form.ts'),
            sourceVoucherCode = this.metaAction.gf('data.form.voucherSource'),
            sourceVoucherTypeId = this.metaAction.gf('data.form.sourceVoucherTypeId'),
            voucher, delText, delSuccessText

        if (sourceVoucherCode) {
            if (sourceVoucherTypeId == 143) {  //TODO  143
                delText = '单据：' + sourceVoucherCode + '是TTK会计填写的凭证，如果驳回，会将对应的流水账同时删除，请确认要驳回吗？'
            }
            else {
                delText = '单据：' + sourceVoucherCode + '将被驳回到业务，当前凭证会被删除。请确认要驳回吗？'
            }
            delSuccessText = '驳回'
        } else {
            delText = '确定删除该凭证?'
            delSuccessText = '删除'
        }
        if (!id || !ts) {
            this.metaAction.toast('info', '请先保存凭证!')
            return
        }
        //让保存按钮置灰，防止并发 置灰
        this.injections.reduce('showBtnState', {
            path: 'data.other.btnStatus',
            value: true
        })
        const ret = await this.metaAction.modal('confirm', {
            title: delSuccessText + '凭证',
            content: delText
        })

        if (ret) {
            const response = await this.webapi.certificate.del({ docId: id, ts: ts })
            this.metaAction.toast('success', '删除凭证成功')

            // 删除或者驳回凭证后自动跳转到下一张凭证
            if (response) {
                this.injections.reduce('loadCertificate', fromJS(response), data.VIEW_STATUS)
            } else {
                this.newCertificate(undefined)
            }
        }
        //让保存按钮置灰，防止并发 置灰
        this.injections.reduce('showBtnState', {
            path: 'data.other.btnStatus',
            value: false
        })
    }

    // 在当前凭证前插入空白凭证 凭证号取点击【插入凭证】前的凭证的凭证号
    insert = async() => {
        let currentCode = this.metaAction.gf('data.form.code'),
            currentDate = this.metaAction.gf('data.form.date')

        if (currentCode == '9999') {
            this.metaAction.toast('error', '凭证号达到9999，不能再增加凭证！')
            return
        }
        const enabledYearMonth = this.getEnabledYearMonth()
        //TTK-3164
        this.checkEditStatusBeforeLoadNewVoucher(() => {
            this.injections.reduce('clearAndNewCertificate', { code: currentCode, voucherDate: currentDate, isInsert: true }, undefined, undefined, enabledYearMonth)
        })
        this.extendAction.gridAction.cellAutoFocus()

    }

    newCertificate = async (subjectList) => {
        let certificateDate = moment(this.metaAction.gf('data.form.date')),
          year = certificateDate.year(),
          period = certificateDate.month() + 1,
          option = {year: year, period: period, isReturnValue: true},
          enabledYearMonth = this.getEnabledYearMonth()

        const initCertificateData = await this.webapi.certificate.init(option)

        if (initCertificateData.result == false && initCertificateData.error && initCertificateData.error.code == '70002') {
            this.metaAction.toast('error', initCertificateData.error.message)
            this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList && subjectList.glAccounts ? subjectList : undefined, undefined, enabledYearMonth)
        } else {
            this.injections.reduce('clearAndNewCertificate', { code: initCertificateData.docCode, voucherDate: initCertificateData.voucherDate }, subjectList && subjectList.glAccounts ? subjectList : undefined, undefined, enabledYearMonth)
            this.extendAction.gridAction.cellAutoFocus()
        }
    }

    loadNextCertificate = () => {
        this.checkEditStatusBeforeLoadNewVoucher(async () => {
            let code = this.metaAction.gf('data.form.code'),
                //新增状态, 当前时间的期间; 查看状态,使用单据时间的期间
                voucherMoment = moment(this.metaAction.gf('data.form.date')),
                currentYear = voucherMoment.year(),
                currentPeriod = voucherMoment.month() + 1,
                editStatus = this.metaAction.gf('data.other.editStatus'),
                isCreate

            if (editStatus == data.ADD_STATUS) {
                isCreate = true
            } else {
                isCreate = false
            }

            const response = await this.webapi.certificate.next({ docCode: code, year: currentYear, period: currentPeriod, isReturnValue: true, isCreate: isCreate })

            if (response.result == false &&
                response.error.code == data.ALREADY_LAST_CERTIFICATE) {
                this.metaAction.toast('warning', response.error.message)
                this.injections.reduce('setNextOrPrivDis', 'nextBtn')
            } else {
                this.injections.reduce('loadCertificate', fromJS(response), data.VIEW_STATUS)
            }
        })
    }

    loadPrevCertificate = () => {
        this.checkEditStatusBeforeLoadNewVoucher(async () => {

            let code = this.metaAction.gf('data.form.code'),
                //新增状态, 当前时间的期间; 查看状态,使用单据时间的期间
                voucherMoment = moment(this.metaAction.gf('data.form.date')),
                currentYear = voucherMoment.year(),
                currentPeriod = voucherMoment.month() + 1,
                editStatus = this.metaAction.gf('data.other.editStatus'),
                isCreate

            if (editStatus == data.ADD_STATUS) {
                isCreate = true
            } else {
                isCreate = false
            }

            const response = await this.webapi.certificate.prev({ docCode: code, year: currentYear, period: currentPeriod, isReturnValue: true, isCreate: isCreate })

            if (response.result == false &&
                response.error.code == data.ALREADY_FIRST_CERTIFICATE) {
                this.metaAction.toast('warning', response.error.message)
                this.injections.reduce('setNextOrPrivDis', 'prevBtn')
            } else {

                this.injections.reduce('loadCertificate', fromJS(response), data.VIEW_STATUS)

            }
        })
    }

    //在加载新单据(空单据)前, 检查并提示保存修改
    checkEditStatusBeforeLoadNewVoucher = async (callBack) => {
        let needSave = false,        //是否需要提示保存: 编辑状态 || 新增状态并填写了分录
            editStatus = this.metaAction.gf('data.other.editStatus')

        if (editStatus == data.EDIT_STATUS) {
            needSave = true
        } else if (editStatus == data.ADD_STATUS) {
            let details = this.metaAction.gf('data.form.details')

            if (details) {
                let emptyItem = Map(data.blankVoucherItem)
                for (let detail of details) {
                    if (detail && !is(detail, emptyItem)) {
                        needSave = true
                        break
                    }
                }
            }
        }

        if (needSave) {
            let handleNegative = () => {
                callBack()
            },
                handleOk = () => {
                    this.save(() => {
                        callBack()
                    })
                }

            const ret = await this.metaAction.modal('confirm', {
                title: '保存',
                content: '您的凭证尚未保存，是否要保存?'
            })

            if (ret) {
                handleOk()
            } else {
                handleNegative()
            }
        } else {
            callBack()
        }
    }

    isCommonTemplateDisabled = () => {
        let editStatus = this.metaAction.gf('data.other.editStatus')
        return (editStatus != data.ADD_STATUS)// 只有新增状态可以引用常用模板
    }

    renderFormHeaderTitleShow = (date) => {
        if (!date) {
            return
        }
        if (typeof date == 'string') {
            date = moment(new Date(date))
        }
        return <span>{`${date.year()}年第${date.month() + 1}期`}</span>
    }

    componentWillUnmount = () => {
        if (sessionStorage.getItem('auxAccountSubjects') != null) {
            sessionStorage.removeItem('auxAccountSubjects')
        }
        if (sessionStorage.getItem('isClickCancelBtn') != null) {
            sessionStorage.removeItem('isClickCancelBtn')
        }
        const win = window
        if (win.removeEventListener) {
            document.getElementById("app").removeEventListener('click', this.windowClick, false)
            document.body.removeEventListener('keydown', this.bodyKeydownEvent, false)
        } else if (win.detachEvent) {
            document.getElementById("app").detachEvent('onclick', this.windowClick)
            document.body.detachEvent('onkeydown', this.bodyKeydownEvent)
        }
    }
}

export default function creator(option) {
    const metaAction = new MetaAction(option),
        extendAction = extend.actionCreator({ ...option, metaAction }),
        o = new action({ ...option, metaAction, extendAction }),
        ret = { ...metaAction, ...extendAction.gridAction, ...o }

    metaAction.config({ metaHandlers: ret })

    return ret
}
