import React from 'react'
import { action as MetaAction, AppLoader } from 'edf-meta-engine'
import {Icon, FormDecorator} from 'edf-component'
import config from './config'
import extend from './extend'

class action {
    constructor(option) {
        this.metaAction = option.metaAction
		this.extendAction = option.extendAction
		this.voucherAction = option.voucherAction
		this.config = config.current
		this.webapi = this.config.webapi
    }

    onInit = ({ component, injections }) => {
        this.extendAction.gridAction.onInit({ component, injections })
        this.component = component
        this.injections = injections
        if (this.component.props.setOkListener) {
            this.component.props.setOkListener(this.onOk)
        }
        if (this.component.props.setCancelLister) {
			this.component.props.setCancelLister(this.onCancel)
		}
        injections.reduce('init')

        this.load()
    }

    load = async () => {
        let option = {}
        option.incomeexpensesTabId = this.component.props.incomeexpensesTabId

        //有record时为编辑状态
        if(this.component.props.record){
            option.form = this.component.props.record
        }

        const initDate = await this.webapi.incomeexpensesCard.init({
            parentId: option.incomeexpensesTabId
        })

        if(initDate){
            option.code = initDate.code
            option.subjectList = initDate.glAccounts
            if(initDate.categories) option.largeClass = initDate.categories
        }

        this.injections.reduce('load', option)
    }

    //编辑大类按钮跳到收支大类列表界面
    editClick = async () => {
        const ret =  await this.metaAction.modal('show', {
			title: '收支大类',
			wrapClassName: 'incomeexpenses-type-list',
			width: 600,
			okText: '确定',
			bodyStyle: {padding: '12px'},
			children: this.metaAction.loadApp('scm-incomeexpenses-type-list', {
                store: this.component.props.store,
                record: this.component.props.record,
                incomeexpensesTabId: this.component.props.incomeexpensesTabId
			}),
        })
        if(ret){}
    }

    //获取类型名称
    getTypeName = () => {
        let incomeexpensesTabId = this.component.props.incomeexpensesTabId
        if(incomeexpensesTabId == '4001001'){
            return '费用类型'
        }else if(incomeexpensesTabId == '3001002'){
            return '收款类型'
        }else if(incomeexpensesTabId == '3001001'){
            return '付款类型'
        }
        return '收入类型'
    }

    //是否显示取消按钮
    getCancleVisible = () => {
        let incomeexpensesTabId = this.component.props.incomeexpensesTabId
        if(incomeexpensesTabId == "2001003" || incomeexpensesTabId == "4001001"){
            return true
        }
        if(this.component.props.record){
            return true
        }
        return false
    }

    //是否显示保存并新增按钮
    getSaveAndNewVisible = () => {
        let incomeexpensesTabId = this.component.props.incomeexpensesTabId
        if(incomeexpensesTabId == "3001002" || incomeexpensesTabId == "3001001"){ //!this.component.props.record
            if(this.component.props.record) return false
            return true
        }
        return false
    }

    filterOption = (inputValue, option) => {
        if (option && option.props && option.props.value) {
            let accountingSubjects = this.metaAction.gf('data.other.defaultProject')
            let itemData = accountingSubjects.find(o => o.get('id') == option.props.value)

            if ((itemData.get('code') && itemData.get('code').indexOf(inputValue) == 0)
                || (itemData.get('gradeName') && itemData.get('gradeName').indexOf(inputValue) != -1)
                || (itemData.get('codeAndName') && itemData.get('codeAndName').indexOf(inputValue) == 0)
                || (itemData.get('helpCode') && itemData.get('helpCode').toUpperCase().indexOf(inputValue.toUpperCase()) != -1)
                || (itemData.get('helpCodeFull') && itemData.get('helpCodeFull').indexOf(inputValue) != -1)) {

                //将滚动条置顶
                let select = document.getElementsByClassName('ant-select-dropdown-menu')
                if (select.length > 0 && select[0].scrollTop > 0) {
                    select[0].scrollTop = 0
                }

                return true
            }
            else {
                return false
            }
        }

        return true
    }

    //取消
    onCancel = () => {
        let isSaveAndNew = this.metaAction.gf('data.other.isSaveAndNew')
        if(isSaveAndNew){
            this.component.props.closeModal(true)
        }else{
            this.component.props.closeModal(false)
        }
    }
    
    //保存
	onOk = async (type) => {
        let save = await this.save(type)
        if(save) this.component.props.closeModal(save)
    }
	
	save = async (type) => {
        let incomeexpensesTabId = this.component.props.incomeexpensesTabId
        let form = this.metaAction.gf('data.form').toJS(), option={}
        let checkArr = [{
			path: 'data.form.code', value: form.code
		}, {
			path: 'data.form.name', value: form.name
        }, {
			path: 'data.form.defaultProject', value: form.defaultProject
        }]

        if(incomeexpensesTabId == '3001002' || incomeexpensesTabId == '3001001'){
            checkArr.push({
                path: 'data.form.receivables', value: form.receivables
            })
        }
		const ok = await this.voucherAction.check(checkArr, this.check)

        if(!ok) return    

        option.code = form.code
        option.name = form.name
        if(form.defaultProject){
            option.accountCode = form.defaultProject.code
            //option.accountTypeId = form.defaultProject.accountTypeId
        }

        let intabId = this.component.props.incomeexpensesTabId,
            record = this.component.props.record, res

        //新增
        if(!record){
            //有大类的时候pid取的是大类的id, 没有大类的时候pid取的是收入类型的id
            if(intabId != '3001002' && intabId != '3001001') {
                option.pid = intabId
            }else{
                option.pid = form.receivables.id
            }
            res = await this.webapi.incomeexpensesCard.create(option)
            if(!res) return false
            this.metaAction.toast('success', '新增成功')
            if(type == 'saveAndNew'){
                this.injections.reduce('saveAndNew', res)
                this.load()
                return false
            }
        //编辑
        }else{
            option.id = this.component.props.record.id
            option.ts = this.component.props.record.ts
            res = await this.webapi.incomeexpensesCard.update(option)
            if(!res) return false
            this.metaAction.toast('success', '编辑成功')
        } 
        if(res) {
            return res
        }else{
            return false
        }
    }	

    //保存校验
    check = async (option) => {
		if (!option || !option.path)
			return

        if (option.path == 'data.form.code') {
			return { errorPath: 'data.other.error.code', message: !(option.value && option.value.trim()) ? '请录入编码' : (option.value.length > 50 ? '编码不能超过50个字' : '') }
		}
		else if (option.path == 'data.form.name') {
			return { errorPath: 'data.other.error.name', message: !(option.value && option.value.trim()) ? '请录入名称' : (option.value.length > 100 ? '名称不能超过100个字' : '') }
		}
        else if (option.path == 'data.form.receivables') {
			return {errorPath: 'data.other.error.receivables', message: option.value ? '' : '请选择大类'}
        }
        else if (option.path == 'data.form.defaultProject') {
			return {errorPath: 'data.other.error.defaultProject', message: option.value ? '' : '请选择默认关联科目'}
		}
	}
    
    fieldChange = (path, value) => {
		this.voucherAction.fieldChange(path, value, this.check)
	}
}

export default function creator(option) {
    const metaAction = new MetaAction(option),
		extendAction = extend.actionCreator({ ...option, metaAction }),
		voucherAction = FormDecorator.actionCreator({ ...option, metaAction }),
		o = new action({ ...option, metaAction, extendAction,voucherAction }),
		ret = { ...metaAction, ...extendAction.gridAction,...voucherAction, ...o }
	
	metaAction.config({ metaHandlers: ret })
	return ret
}