import React from 'react'
import { action as MetaAction, AppLoader } from 'edf-meta-engine'
import { List, fromJS } from 'immutable'
import moment from 'moment'
import config from './config'
import { trimLeft, trimRight, trim } from '../../../../../utils/string/index'
import utils from 'edf-utils'
import {fetch} from 'edf-utils'

import { TableOperate, Table, Select, Button, Modal, Input, Number, Checkbox, Icon, Popconfirm, FormDecorator, LoadingMask, Upload} from 'edf-component'

const Option = Select.Option;

const ExtendEditableSelectCell = ({ dataSource, onChange, value, columnName, onClick, saveNewVlue, handleFoucus, disabled}) => {
  let productChildren = []
  //新增按钮 实现新增档案

  if (dataSource) {
    dataSource.forEach((item,index) => {
        if(typeof item == 'object') {
            productChildren.push(<Option title={item.title} value={item.fileNameInDto}>{item.title}</Option>)
        } else {
            productChildren.push(<Option title={item} value={item} >{item}</Option>)
        }
    })
  }
  return (
    <div style={{width: '100%',height: '26px'}}>
      <Select style={{ width: '100%' }}
      onChange={(value) => onChange(value)}
      value={(value && value.id) ? value.id : value}
      filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
      disabled = {disabled}
      dropdownClassName='projectadjust-select-dropDown'
       onFocus={handleFoucus}>
        {productChildren}
      </Select>
    </div>
  )
}

const EditableCheckbox = ({ checked, onChange}) => {
	return (
		<div style={{'display': 'block', 'width': '100%','textAlign': 'center'}}>
			<Checkbox 
				onChange={onChange} 
				checked={checked}
			></Checkbox>
		</div>
	)
}

class action {
    constructor(option) {
        this.metaAction = option.metaAction
        this.voucherAction = option.voucherAction
        this.config = config.current
		this.webapi = this.config.webapi
		this.beforeLoad = option.voucherAction.beforeUpload
    }

    onInit = ({ component, injections }) => {
        this.voucherAction.onInit({ component, injections })
        this.component = component
        this.injections = injections

        if (this.component.props.setOkListener) {
            this.component.props.setOkListener(this.onOk)
        }
        injections.reduce('init', {
            status: this.component.props.status,
            initData: this.component.props.initData
        })

        this.load()
    }

    load = async () => {
        let reportCode = this.metaAction.gf('data.form.activeTab')?this.metaAction.gf('data.form.activeTab'):'balancesheet'
        this.queryTableList(reportCode)
        document.querySelector('.tranreport-projectadjust-modal .ant-modal-footer .ant-btn-primary').disabled = true
	}
	
	//start
    rptTypeChange = (path, value) => {
        this.metaAction.sf(path, value)
    }
    
    queryTableList = async (reportCode) => {
        let res,
            params = this.metaAction.gf('data.params') ? {
                ...this.metaAction.gf('data.params').toJS()
            }:{}
        this.metaAction.sf('data.other.loading',true)
        res = await this.webapi.projectadjust.init( params )
        this.metaAction.sf('data.other.loading',false)
        if( this.metaAction.gf('data.other.fromPromp') ) {
            let firstReportCode

            if(res){
                res.map( item => {
                    if(!firstReportCode && ( (item.headList&&item.headList.length) || (item.detailList&&item.detailList.length)  ) ) {
                        firstReportCode = item.reportCode.toLocaleLowerCase()
                    }
                })
            }
            reportCode = firstReportCode?firstReportCode:'balancesheet'
        }
        this.injections.reduce('load', res, reportCode)
    }

	// new start
	//获取需要编辑的列表
	getCurList = (tableType) => {
		let activeTab = this.metaAction.gf('data.form.activeTab')?this.metaAction.gf('data.form.activeTab'):'balancesheet',
			list,
            path,
            dataSourcePath,
            params= {}
        path = `data.${activeTab}${tableType}List`
        dataSourcePath = `data.${activeTab}${tableType}DataSource`
		params.path = path
		params.list = this.metaAction.gf( path ) ? this.metaAction.gf( path ).toJS() : []
		params.dataSource = this.metaAction.gf( dataSourcePath ) ? this.metaAction.gf( dataSourcePath ).toJS() : []
		return params
    }
    
    //判断该列单元格是否是select框
    renderCell = (columnName, index, value, tableType) => {
		let editable = true
		let activeTab = this.metaAction.gf('data.form.activeTab')?this.metaAction.gf('data.form.activeTab'):'balancesheet',
			params = this.getCurList(tableType),
			list = params.list
        if( list[index]['empty'] ) {
            return this.renderTextCell(columnName, index, value)			
		} else {
            if( columnName == 'targetame' ) {
                return (
                  <ExtendEditableSelectCell
                    editable={editable}
                    onChange={(value) => this.handleSelectChange(value, columnName, index, params.dataSource, tableType)}
                    dataSource={params.dataSource}
                    value={value}
                    columnName={columnName}
                    onClick={() => {this.handleOnClickAdd(columnName, index, tableType)}}
                    disabled = { list[index]['ignore'] }
                  />
                )
            } else if( columnName == 'ignore' ) {
				return (<EditableCheckbox
					checked = {value}
					onChange={(value) => this.handleChange(value, columnName, index, tableType)}
				/>)
            }
        }
        return this.renderTextCell(columnName, index, value, list[index]['ignore'] , list[index]['new'])
	}

    renderTextCell = (columnName, index, value, ignore, newStr) => {
        return (<div className={ ignore?'tranreport-projectadjust-cell cell-color-grid':'tranreport-projectadjust-cell'}>
            <span title={value}>{value}</span> {(columnName=='sourceName' && newStr==true)? <img src={require('./img/new.gif')} style={{width:'29px',height:'11px','marginTop':'5px','minWidth':'29px'}} />:''}
        </div>)
    }

	//多选框
	handleChange = (value, columnName, index, tableType) => {//焦点离开输入框触发
        let params = this.getCurList(tableType),
			list = fromJS(params.list),
			oldList = params.list,
			path = params.path
        let values
    
        list = list.update(index, item => item.set( 'ignore', value.target.checked ))
        list = list.update(index, item => item.set( 'targetame', undefined ))
        list = list.update(index, item => item.set( 'targeFieldName', undefined ))
        this.updateTableItem( tableType, list.toJS(), oldList, path, index )
    }

    handleSelectChange = async (value, columnName, index, dataSource, tableType) => {//选择框值改变触发
		let params = this.getCurList(tableType),
            list = fromJS(params.list),
            oldList = params.list,
			path = params.path
        if( tableType == 'Col' ) {
            const selectObj = dataSource.filter((item) => item.fileNameInDto == value) 
            list = list.update(index, item => item.set('targetame', selectObj[0].title ))
            list = list.update(index, item => item.set('targeFieldName', selectObj[0].fileNameInDto ))
        } else if( tableType == 'Row' ) {
            list = list.update(index, item => item.set(columnName, value ))
        
        }
        //改值实时调接口
        this.updateTableItem( tableType, list.toJS(), oldList, path, index )
    }

    updateTableItem = async ( tableType, list, oldList, path, index) => {
        let activeTab = this.metaAction.gf('data.form.activeTab')?this.metaAction.gf('data.form.activeTab'):'balancesheet'
        
        document.querySelector('.tranreport-projectadjust-modal .ant-modal-footer .ant-btn-primary').disabled = false
        this.metaAction.sf(`data.other.${activeTab}EditStatus`, true)
        this.metaAction.sf(path, fromJS(list))
    }

    handleOnClickAdd = async (value, rowIndex, tableType) => {
        let params = this.getCurList(tableType),
			list = params.list,
			path = params.path
	}

    onOk = async () => {
        return await this.save()
    }

    save = async () => {
        let checkResponse = this.checkLists(),
        res2
        if (!checkResponse.responseStatus) {
            let res1 =  await this.metaAction.modal('confirm', {
                content: checkResponse.messageInfo1.join('、') +'存在无对应关系的原表'+checkResponse.messageInfo2.join('、')+'，将被忽略掉。您可以取消忽略并为其选择匹配的局端'+checkResponse.messageInfo2.join('、')+'。',
                className: 'tranreport-prompt-modal'
            })
            if(!res1) {
                return false
            }
        } 
        let params = this.metaAction.gf('data.params') ? {
            calcBalanceSheet: !!this.metaAction.gf('data.other.balancesheetEditStatus'),
	        calcProfitStatement: !!this.metaAction.gf('data.other.profitstatementEditStatus'),
            calcCashFlowStatement: !!this.metaAction.gf('data.other.cashflowstatementEditStatus'),
            calcSumRow:true,
            headList: checkResponse.colSuccessList,
            detailList: checkResponse.rowSuccessList,
            ...this.metaAction.gf('data.params').toJS()
        }:{}

        let res = await this.webapi.projectadjust.matchingBatch( params )
        if(res) {
            delete params.headList
            delete params.detailList
            res2 = await this.webapi.projectadjust.calcReport( params )
            if(res2) {
                return res2
            }
        }
		// return res
	}

	checkLists = () => {
        let tableName = [ 'balancesheet', 'profitstatement', 'cashflowstatement' ],
            checkResponse = true,
            balancesheetColList = this.metaAction.gf('data.balancesheetColList') ? this.metaAction.gf('data.balancesheetColList').toJS():[],
			balancesheetRowList = this.metaAction.gf('data.balancesheetRowList') ? this.metaAction.gf('data.balancesheetRowList').toJS():[],
			cashflowstatementColList = this.metaAction.gf('data.cashflowstatementColList') ? this.metaAction.gf('data.cashflowstatementColList').toJS():[],
			cashflowstatementRowList = this.metaAction.gf('data.cashflowstatementRowList') ? this.metaAction.gf('data.cashflowstatementRowList').toJS():[],
			profitstatementColList = this.metaAction.gf('data.profitstatementColList') ? this.metaAction.gf('data.profitstatementColList').toJS():[],
            profitstatementRowList = this.metaAction.gf('data.profitstatementRowList') ? this.metaAction.gf('data.profitstatementRowList').toJS():[],
            checkData = [],
            colIgnoreList = [],rowIgnoreList =[],messageInfo1 = [],messageInfo2 = [],
            colSuccessList = [],
            rowSuccessList = [],
            editItems = []

        tableName.map( item => {
            let rescol={},resrow={}
            if( this.metaAction.gf(`data.${item}ColState`) ) {
                let colList = this.metaAction.gf(`data.${item}ColList`).toJS()
                    
                colList = colList.slice( 0, this.metaAction.gf(`data.${item}ColListLength`) )
                rescol = this.checkListData( colList ,'col' )
                if(!rescol.state) {
                    colIgnoreList = colIgnoreList.concat( rescol.data )
                }
                colSuccessList = colSuccessList.concat( rescol.successData )
                if(checkResponse) {
                    checkResponse = rescol.state
                }
            }
            if( this.metaAction.gf(`data.${item}RowState`) ) {
                let rowList = this.metaAction.gf(`data.${item}RowList`).toJS()
                rowList = rowList.slice( 0, this.metaAction.gf(`data.${item}RowListLength`) )
                resrow = this.checkListData( rowList ,'row' )
                if(!resrow.state) {
                    rowIgnoreList = rowIgnoreList.concat( resrow.data )
                }
                rowSuccessList = rowSuccessList.concat( resrow.successData )
                if(checkResponse) {                    
                    checkResponse = resrow.state
                }
            }
            if(resrow.state === false || rescol.state === false) {
                editItems.push(item)
                messageInfo1.push(
                    (item == 'balancesheet'?'资产负债表':( item == 'profitstatement' ? '利润表' : '现金流量表') )
                )
            }
        })
        if(rowIgnoreList.length) {//有没忽略的行数据 提示信息中加 行
            messageInfo2.push('行')
        }
        if(colIgnoreList.length) {//有没忽略的列数据 提示信息中加 列
            messageInfo2.push('列')
        }
        return {
            responseStatus:checkResponse,
            colIgnoreList,
            rowIgnoreList,
            colSuccessList: colSuccessList.concat( colIgnoreList ),
            rowSuccessList: rowSuccessList.concat( rowIgnoreList ),
            editItems,
            messageInfo1,//提示信息包含的报表名
            messageInfo2//提示信息包含的行或列
        }
    }
    
    checkListData = (list, type) => {
        let checkResponse = true,
            arr = [],
            successArr = []
        list.map( item => {
            if( !item.targetame && !item.ignore ) {
                checkResponse = false
                delete item.new
                delete item.ts
                delete item.orgId
                item.ignore = 1
                if(type == 'col') {
                    item.targeFieldName=null
                    item.targetame=null
                } else if(type == 'row') {
                    item.targetame=null
                }
                arr.push(item)
            } else {
                item.ignore = item.ignore?1:0
                delete item.ts
                delete item.orgId
                delete item.new
                if(type == 'col') {
                    if(!item.targeFieldName) {
                        item.targeFieldName=null
                    }
                    if(!item.targetame) {
                        item.targetame=null
                    }
                } else if(type == 'row') {
                    if(!item.targetame) {
                        item.targetame=null
                    }
                }
                successArr.push(item)                
            }
        })
        return {
            state: checkResponse,
            data: arr,
            successData: successArr
        }
    }

    getTableTitle = () => {
        return <p style={{'marginBottom': '0px'}} style={{width: '100%','marginBottom': 0}}>
            行：<span className="tranreport-projectadjust-table-right-title" style={{'color': '#FF7E3B','float': 'right','fontSize':'12px'}}>注：下方表格只显示不一致的行。</span>
        </p>
    }
}

export default function creator(option) {
    const metaAction = new MetaAction(option),
        voucherAction = FormDecorator.actionCreator({ ...option, metaAction }),
        o = new action({ ...option, metaAction, voucherAction }),
        ret = { ...metaAction, ...voucherAction, ...o }

    metaAction.config({ metaHandlers: ret })

    return ret
}